@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4156 $
+//  $Revision: 5008 $
 //
-//  $LastChangedDate: 2012-04-10 10:13:18 -0700 (Tue, 10 Apr 2012) $
+//  $LastChangedDate: 2013-10-28 14:12:33 -0700 (Mon, 28 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -51,11 +51,12 @@
 #include <Base/cmtkImageOperationFlip.h>
 #include <IO/cmtkImageOperationApplyMask.h>
 #include <Base/cmtkImageOperationErodeDilate.h>
+#include <Base/cmtkImageOperationErodeDilateDistance.h>
 #include <Base/cmtkImageOperationBoundaryMap.h>
 #include <Base/cmtkImageOperationConnectedComponents.h>
 #include <Base/cmtkImageOperationDownsample.h>
+#include <Base/cmtkImageOperationResampleIsotropic.h>
 #include <Base/cmtkImageOperationHistogramEqualization.h>
-#include <Base/cmtkImageOperationHoughTransformSphere.h>
 #include <Base/cmtkImageOperationCropRegion.h>
 #include <Base/cmtkImageOperationCropThreshold.h>
 #include <Base/cmtkImageOperationScaleToRange.h>
@@ -66,7 +67,6 @@
 #include <Base/cmtkImageOperationMapValues.h>
 #include <Base/cmtkImageOperationReplace.h>
 #include <Base/cmtkImageOperationRegionFilter.h>
-#include <Base/cmtkImageOperationMedialSkeleton.h>
 #include <Base/cmtkImageOperationGaussFilter.h>
 #include <Base/cmtkImageOperationLaplaceFilter.h>
 #include <Base/cmtkImageOperationDistanceMap.h>
@@ -85,8 +85,8 @@
 int
 doMain( const int argc, const char* argv[] )
 {
-  const char* imagePathIn = NULL;
-  const char* imagePathOut = NULL;
+  std::string imagePathIn;
+  std::string imagePathOut;
 
 #ifdef CMTK_USE_SQLITE
   const char* updateDB = NULL;
@@ -168,15 +168,16 @@ doMain( const int argc, const char* argv[] )
     
     cl.BeginGroup( "Morphological", "Morphological Operations" );
     cl.AddCallback( Key( "revert" ), &cmtk::ImageOperationRevert::New, "Revert a binary mask, i.e., exchange foreground and background." );
-    cl.AddCallback( Key( "erode" ), &cmtk::ImageOperationErodeDilate::NewErode, "Morphological erosion operator" );
-    cl.AddCallback( Key( "dilate" ), &cmtk::ImageOperationErodeDilate::NewDilate, "Morphological dilation operator" );
+    cl.AddCallback( Key( "erode" ), &cmtk::ImageOperationErodeDilate::NewErode, "Morphological erosion operator (by pixels)" );
+    cl.AddCallback( Key( "dilate" ), &cmtk::ImageOperationErodeDilate::NewDilate, "Morphological dilation operator (by pixels)" );
+    cl.AddCallback( Key( "erode-distance" ), &cmtk::ImageOperationErodeDilateDistance::NewErode, "Morphological erosion operator (by distance). Often preferable for anisotropic data." );
+    cl.AddCallback( Key( "dilate-distance" ), &cmtk::ImageOperationErodeDilateDistance::NewDilate, "Morphological dilation operator (by distance). Oftern preferable for anisotropic data." );
     cl.AddCallback( Key( "connected-components" ), &cmtk::ImageOperationConnectedComponents::New, "Create connected components map with regions numbered by decreasing component size" );
     cl.AddCallback( Key( "boundary-map" ), &cmtk::ImageOperationBoundaryMap::New, "Create boundary map" );
     cl.AddCallback( Key( "multi-boundary-map" ), &cmtk::ImageOperationBoundaryMap::NewMulti, "Create multi-valued boundary map" );
 
     cl.AddCallback( Key( "distance-map" ), &cmtk::ImageOperationDistanceMap::NewUnsigned, "Compute unsigned Euclidean distance map. Input image is interpreted as binary mask." );
     cl.AddCallback( Key( "signed-distance-map" ), &cmtk::ImageOperationDistanceMap::NewSigned, "Compute signed (inside=negative, outside=positive) Euclidean distance map" );
-    cl.AddCallback( Key( "medial-skeleton" ), &cmtk::ImageOperationMedialSkeleton::New, "Compute n-dimensional medial skeleton of binary mask. Argument must be either 1 or 2." );
     cl.EndGroup();
 
     cl.BeginGroup( "Filtering", "Filter Operations" );
@@ -208,14 +209,11 @@ doMain( const int argc, const char* argv[] )
     cl.AddCallback( Key( "laplace-filter" ), &cmtk::ImageOperationLaplaceFilter::New, "Filter image with edge-enhancing Laplacian kernel." );
     cl.EndGroup();
     
-    cl.BeginGroup( "Feature", "Feature Detection Operations" );
-    cl.AddCallback( Key( "hough-sphere" ), &cmtk::ImageOperationHoughTransformSphere::New, "Hough transform to detect sphere with a given radius. "
-		    "The radius in world units (typically mm) is provided as an argument to this operation." );
-    cl.EndGroup();
-    
     cl.BeginGroup( "Grid", "Grid Operations" );
     cl.AddCallback( Key( "downsample-select" ), &cmtk::ImageOperationDownsample::NewSelect, "Downsample image by pixel selection using per-axis factors 'Fx,Fy,Fz' or using single factor 'Fxyz'" );
     cl.AddCallback( Key( "downsample-average" ), &cmtk::ImageOperationDownsample::NewAverage, "Downsample image by averaging using per-axis factors 'Fx,Fy,Fz' or using single factor 'Fxyz'" );
+    cl.AddCallback( Key( "resample" ), &cmtk::ImageOperationResampleIsotropic::New, "Resample image to near-isotropic pixels while preserving the image field-of-view. "
+		    "Takes one argument, the target resolution in world units [e.g., mm]" );
     cl.AddCallback( Key( "crop-by-index" ), &cmtk::ImageOperationCropRegion::New, "Crop image to a region specified by a set of six grid index coordinates given as comma-separated integers x0,y0,z0,x1,y1,z1" );
     cl.AddCallback( Key( "crop-by-threshold" ), &cmtk::ImageOperationCropThreshold::New, "Crop image to region determined via a given threshold. "
 		    "The resulting image will contain all pixels larger than the given parameter." );
