@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2012 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4555 $
+//  $Revision: 5010 $
 //
-//  $LastChangedDate: 2012-10-15 16:47:10 -0700 (Mon, 15 Oct 2012) $
+//  $LastChangedDate: 2013-11-07 16:52:03 -0800 (Thu, 07 Nov 2013) $
 //
-//  $LastChangedBy: torsten_at_home $
+//  $LastChangedBy: torstenrohlfing $
 //
 */
 
@@ -116,6 +116,7 @@ std::vector<std::string> SearchRootDirVector;
 
 std::ofstream cnull( "/dev/null" );
 
+bool Progress = true;
 bool Recursive = false;
 
 /// Enum type to select sort order key.
@@ -132,6 +133,7 @@ typedef enum
 SortKeyEnum SortFiles = SORT_INSTANCE_NUMBER;
 
 bool WriteXML = false;
+bool IncludeIdentifiers = false;
 
 bool WriteSingleSlices = false;
 
@@ -144,7 +146,7 @@ using cmtk::ImageFileDICOM;
 using cmtk::ImageStackDICOM;
 
 /// Selector for embedded image information.
-ImageStackDICOM::EmbedInfoEnum EmbedInfo = ImageStackDICOM::EMBED_STUDYID_STUDYDATE;
+ImageStackDICOM::EmbedInfoEnum EmbedInfo = ImageStackDICOM::EMBED_NONE;
 
 /// Class handling a list of image stacks, i.e., volumes.
 class VolumeList : 
@@ -154,6 +156,9 @@ public:
   /// This class.
   typedef VolumeList Self;
 
+  /// Constructor.
+  VolumeList( const cmtk::Types::Coordinate tolerance = 0 /*!< Tolerance for floating point comparisons, e.g., when testing for uniform pixel/slice spacings.*/ ) : m_Tolerance( tolerance ) {}
+
   /// Build a volume list by traversing a directory.
   void AppendFromDirectory( const std::string& path, const char *wildcard );
   
@@ -161,6 +166,9 @@ public:
   void WriteVolumes();
 
 private:
+  /// Stored floating point tolerance.
+  cmtk::Types::Coordinate m_Tolerance;
+  
   /// Add a new image file to the correct stack or create a new stack.
   void AddImageFile( ImageFileDICOM::SmartConstPtr& newImage );
 };
@@ -168,7 +176,10 @@ private:
 std::string
 PutNumberAndSanitize( const std::string& path, const std::string& numberString )
 {
-  return cmtk::StrReplace( cmtk::StrReplace( path, "%n", numberString ), "%N", "-" + numberString );
+  if ( numberString.empty() )
+    return cmtk::StrReplace( cmtk::StrReplace( path, "%n", "" ), "%N", "" );
+  else
+    return cmtk::StrReplace( cmtk::StrReplace( path, "%n", numberString ), "%N", "-" + numberString );
 }
 
 void
@@ -259,9 +270,9 @@ VolumeList::WriteVolumes()
 
       cmtk::UniformVolume::SmartConstPtr volume = it->second[0]->WriteImage( uniquePath.c_str(), EmbedInfo );
 
-      if ( WriteXML )
+      if ( volume && WriteXML )
 	{
-	it->second[0]->WriteXML( (uniquePath+".xml").c_str(), *volume );
+	it->second[0]->WriteXML( (uniquePath+".xml").c_str(), *volume, IncludeIdentifiers );
 	}
       }
     else
@@ -278,9 +289,9 @@ VolumeList::WriteVolumes()
 
 	cmtk::UniformVolume::SmartConstPtr volume = it->second[i]->WriteImage( uniquePath.c_str(), EmbedInfo );
 
-	if ( WriteXML )
+	if ( volume && WriteXML )
 	  {
-	  it->second[i]->WriteXML( (uniquePath + ".xml").c_str(), *volume );
+	  it->second[i]->WriteXML( (uniquePath + ".xml").c_str(), *volume, IncludeIdentifiers );
 	  }
 	}
       }
@@ -309,7 +320,7 @@ VolumeList::AddImageFile( ImageFileDICOM::SmartConstPtr& newImage )
       }
     }    
 
-  ImageStackDICOM::SmartPtr newImageStack( new ImageStackDICOM );
+  ImageStackDICOM::SmartPtr newImageStack( new ImageStackDICOM( Tolerance ) );
   newImageStack->AddImageFile( newImage );
   push_back( newImageStack );    
 }
@@ -317,21 +328,18 @@ VolumeList::AddImageFile( ImageFileDICOM::SmartConstPtr& newImage )
 void
 VolumeList::AppendFromDirectory( const std::string& path, const char *wildcard )
 {
-  static int progress = 0;
-  const char progress_chars[] = "-\\|/";
-
-  char fullname[PATH_MAX];
-
   std::vector<ImageFileDICOM::SmartConstPtr> fileList;
 
 #ifdef _MSC_VER
+  static int progress = 0;
+  const char progress_chars[] = "-\\|/";
+
   WIN32_FIND_DATA fData;
-  char pattern[PATH_MAX];
-  snprintf( pattern, sizeof( pattern ), "%s\\%s", path.c_str(), wildcard );
-  HANDLE hFind = FindFirstFile( pattern, &fData);
+  const std::string pattern = path + "\\" + wildcard;
+  HANDLE hFind = FindFirstFile( pattern.c_str(), &fData);
   do
     {
-    snprintf( fullname, sizeof( fullname ), "%s\\%s", path.c_str(), fData.cFileName );
+    const std::string fullname = path + "\\" + fData.cFileName;
     if ( fData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
       {
       if ( Recursive && (fData.cFileName[0] != '.') )
@@ -351,7 +359,10 @@ VolumeList::AppendFromDirectory( const std::string& path, const char *wildcard )
 	  fileList.push_back( newFile );
 	  }
 
-	(cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
+	if ( Progress )
+	  {
+	  (cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
+	  }
 	}
       catch ( ... )
 	{
@@ -364,18 +375,20 @@ VolumeList::AppendFromDirectory( const std::string& path, const char *wildcard )
   DIR *dir_pointer = opendir ( path.c_str() );
   if ( dir_pointer != NULL ) 
     {
+    static int progress = 0;
+    const char progress_chars[] = "-\\|/";
     struct dirent *entry_pointer;
 
     while ( (entry_pointer = readdir(dir_pointer)) ) 
       {
-      strcat( strcat( strcpy( fullname, path.c_str() ), "/"), entry_pointer->d_name );
+      const std::string fullname = path + "/" + entry_pointer->d_name;
+
       struct stat entry_status;
-      if ( !stat(fullname, &entry_status) ) 
+      if ( !stat(fullname.c_str(), &entry_status) ) 
 	{
 	if ( S_ISDIR( entry_status.st_mode ) && Recursive && (entry_pointer->d_name[0] != '.') ) 
 	  {
-	  strcat( fullname, "/" );
-	  this->AppendFromDirectory( fullname, wildcard );
+	  this->AppendFromDirectory( fullname + "/" , wildcard );
 	  } 
 	else
 	  {
@@ -391,7 +404,10 @@ VolumeList::AppendFromDirectory( const std::string& path, const char *wildcard )
 		fileList.push_back( newFile );
 		}
 	      
-	      (cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
+	      if ( Progress )
+		{
+		(cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
+		}
 	      }
 	    catch ( ... )
 	      {
@@ -429,7 +445,10 @@ VolumeList::AppendFromDirectory( const std::string& path, const char *wildcard )
       }
     }
 
-  cmtk::StdErr << "\r";
+  if ( Progress )
+    {
+    cmtk::StdErr << "\r";
+    }
 }
 
 
@@ -465,6 +484,8 @@ doMain ( const int argc, const char *argv[] )
     cl.SetProgramInfo( cmtk::CommandLine::PRG_DESCR, "Combine sets of DICOM slices to 3D image stacks" );
 
     typedef cmtk::CommandLine::Key Key;
+    cl.AddSwitch( Key( "no-progress" ), &Progress, false, "Disable progress reporting." );
+
     cl.BeginGroup( "Input", "Input Options");
     cl.AddSwitch( Key( 'r', "recurse" ), &Recursive, true, "Recurse into directories" );
     cl.EndGroup();
@@ -480,6 +501,7 @@ doMain ( const int argc, const char *argv[] )
 		  "%T (RawDataType - vendor-specific, currently GE MRI only)" );
 
     cl.AddSwitch( Key( 'x', "xml" ), &WriteXML, true, "Write XML sidecar file for each created image." );
+    cl.AddSwitch( Key( "include-identifiers" ), &IncludeIdentifiers, true, "Include potentially protected identifying information (e.g., UIDs, device serial numbers, dates) in the created XML sidecar files." );
 
     cmtk::CommandLine::EnumGroup<cmtk::ImageStackDICOM::EmbedInfoEnum>::SmartPtr embedGroup = cl.AddEnum( "embed", &EmbedInfo, "Embed DICOM information into output images as 'description' (if supported by output file format)." );
     embedGroup->AddSwitch( Key( "StudyID_StudyDate" ), cmtk::ImageStackDICOM::EMBED_STUDYID_STUDYDATE, "StudyID, tag (0020,0010), then underscore, followed by StudyDate, tag (0008,0020). "
@@ -526,7 +548,7 @@ doMain ( const int argc, const char *argv[] )
     dcmDataDict.unlock();
     }
   
-  VolumeList volumeList;
+  VolumeList volumeList( Tolerance );
   for ( std::vector<std::string>::const_iterator it = SearchRootDirVector.begin(); it != SearchRootDirVector.end(); ++it )
     {
     volumeList.AppendFromDirectory( *it, "*" );
