@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4076 $
+//  $Revision: 4834 $
 //
-//  $LastChangedDate: 2012-03-23 13:31:42 -0700 (Fri, 23 Mar 2012) $
+//  $LastChangedDate: 2013-09-12 15:15:51 -0700 (Thu, 12 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -47,19 +47,19 @@ int
 doMain
 ( const int argc, const char *argv[] )
 {
-  const char* inputImagePath1 = NULL;
-  const char* inputImagePath2 = NULL;
+  std::string inputImagePath1;
+  std::string inputImagePath2;
 
-  const char* outputImagePath1 = NULL;
-  const char* outputImagePath2 = NULL;
+  std::string outputImagePath1;
+  std::string outputImagePath2;
 
-  const char* outputDField = NULL;
+  std::string outputDField;
 
-  const char* writeJacobianPath1 = NULL;
-  const char* writeJacobianPath2 = NULL;  
+  std::string writeJacobianPath1;
+  std::string writeJacobianPath2;  
 
   byte phaseEncodeDirection = 1;
-
+  bool flipPEPolar = true;
   bool initShiftCentersOfMass = true;
 
   double smoothnessConstraintWeight = 0;
@@ -83,6 +83,8 @@ doMain
     phaseEncodeGroup->AddSwitch( Key( 'y', "phase-encode-ap" ), 1, "Anterior/posterior phase encoding (this is the most common case)" );
     phaseEncodeGroup->AddSwitch( Key( 'z', "phase-encode-is" ), 2, "Top/bottom phase encoding (this is rare)" );
     phaseEncodeGroup->AddSwitch( Key( 'x', "phase-encode-lr" ), 0, "Lateral, left/right phase encoding (this is extremely rare)" );
+    cl.AddSwitch( Key( "no-flip" ), &flipPEPolar, false, "Use this switch is the reverse phase-encoded image does not need to be flipped prior to unwarping. If normal and reverse phase-encoded image display in the same gross "
+		  "orientation in 'triplanar', then flipping is not necessary and must be turned off using this switch." );
     cl.EndGroup();
 
     cl.BeginGroup( "Optimization", "Optimization Parameters" );    
@@ -120,7 +122,8 @@ doMain
   cmtk::UniformVolume::SmartPtr inputImage1 = cmtk::VolumeIO::ReadOriented( inputImagePath1 );
   cmtk::UniformVolume::SmartPtr inputImage2 = cmtk::VolumeIO::ReadOriented( inputImagePath2 );
 
-  inputImage2->ApplyMirrorPlane( phaseEncodeDirection );
+  if ( flipPEPolar )
+    inputImage2->ApplyMirrorPlane( phaseEncodeDirection );
 
   cmtk::EchoPlanarUnwarpFunctional func( inputImage1, inputImage2, phaseEncodeDirection, initShiftCentersOfMass );
 
@@ -131,16 +134,16 @@ doMain
   cmtk::VolumeIO::Write( *func.GetCorrectedImage( +1 ), outputImagePath1 );
   cmtk::VolumeIO::Write( *func.GetCorrectedImage( -1 ), outputImagePath2 );
 
-  if ( outputDField )
+  if ( !outputDField.empty() )
     {
     cmtk::DeformationField::SmartPtr dfield( func.GetDeformationField( +1 ) );
     cmtk::XformIO::Write( dfield, outputDField );
     }
     
-  if ( writeJacobianPath1 )
+  if ( !writeJacobianPath1.empty() )
     cmtk::VolumeIO::Write( *func.GetJacobianMap( +1 ), writeJacobianPath1 );
 
-  if ( writeJacobianPath2 )
+  if ( !writeJacobianPath2.empty() )
     cmtk::VolumeIO::Write( *func.GetJacobianMap( -1 ), writeJacobianPath2 );
 
   return 0;
