@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4427 $
+//  $Revision: 4885 $
 //
-//  $LastChangedDate: 2012-06-12 11:23:22 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2013-09-27 21:04:59 -0700 (Fri, 27 Sep 2013) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -95,7 +95,7 @@ const char* cropReferenceFromIndex = NULL;
 const char* cropReferenceToIndex = NULL;
 
 cmtk::UniformVolume::SmartPtr
-MakeDownsampled( cmtk::UniformVolume::SmartConstPtr& image, const int downsample, const cmtk::Types::Coordinate smoothSigmaFactor )
+MakeDownsampled( cmtk::UniformVolume::SmartConstPtr& image, const int downsample )
 {
   if ( downsampleWithAverage )
     return cmtk::UniformVolume::SmartPtr( image->GetDownsampled( downsample * image->GetMinDelta() ) );
@@ -173,14 +173,14 @@ DoRegistration()
     functional->ClearAllChannels();
     for ( std::list<cmtk::UniformVolume::SmartPtr>::iterator it = refChannelList.begin(); it != refChannelList.end(); ++it )
       {
-      cmtk::UniformVolume::SmartPtr image = MakeDownsampled( (*it), downsample, smoothSigmaFactor );
+      cmtk::UniformVolume::SmartPtr image = MakeDownsampled( (*it), downsample );
       image->CopyMetaInfo( **it, cmtk::META_FS_PATH );
       functional->AddReferenceChannel( image );
       }
 
     for ( std::list<cmtk::UniformVolume::SmartPtr>::iterator it = fltChannelList.begin(); it != fltChannelList.end(); ++it )
       {
-      cmtk::UniformVolume::SmartPtr image = MakeDownsampled( (*it), downsample, smoothSigmaFactor );
+      cmtk::UniformVolume::SmartPtr image = MakeDownsampled( (*it), downsample );
       image->CopyMetaInfo( **it, cmtk::META_FS_PATH );
       functional->AddFloatingChannel( image );
       }
@@ -286,7 +286,7 @@ doMain( const int argc, const char* argv[] )
   if ( cropReferenceFromIndex )
     {
     int xyz[3];
-    if ( 3 != sscanf( cropReferenceFromIndex, "%d,%d,%d", &xyz[0], &xyz[1], &xyz[2] ) )
+    if ( 3 != sscanf( cropReferenceFromIndex, "%6d,%6d,%6d", &xyz[0], &xyz[1], &xyz[2] ) )
       {
       cmtk::StdErr << "ERROR: reference crop from index could not parse index '" << cropReferenceFromIndex << "' as valid x,y,z index.\n";
       throw cmtk::ExitException( 1 );
@@ -300,9 +300,9 @@ doMain( const int argc, const char* argv[] )
   if ( cropReferenceToIndex )
     {
     int xyz[3];
-    if ( 3 != sscanf( cropReferenceToIndex, "%d,%d,%d", &xyz[0], &xyz[1], &xyz[2] ) )
+    if ( 3 != sscanf( cropReferenceToIndex, "%6d,%6d,%6d", &xyz[0], &xyz[1], &xyz[2] ) )
       {
-      cmtk::StdErr << "ERROR: reference crop to index could not parse index '" << cropReferenceFromIndex << "' as valid x,y,z index.\n";
+      cmtk::StdErr << "ERROR: reference crop to index could not parse index '" << cropReferenceToIndex << "' as valid x,y,z index.\n";
       throw cmtk::ExitException( 1 );
       }
     for ( std::list<cmtk::UniformVolume::SmartPtr>::iterator refIt = refChannelList.begin(); refIt != refChannelList.end(); ++refIt )
