@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4427 $
+//  $Revision: 5091 $
 //
-//  $LastChangedDate: 2012-06-12 11:23:22 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2013-12-03 23:42:09 -0800 (Tue, 03 Dec 2013) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -79,10 +79,10 @@ CallbackTargetVolume( const char* arg )
   float gridDelta[3] = { 0, 0, 0 };
   float gridOffset[3] = { 0, 0, 0 };
 
-  const size_t numArgs = sscanf( arg, "%d,%d,%d:%f,%f,%f:%f,%f,%f", gridDims, gridDims+1, gridDims+2, gridDelta, gridDelta+1, gridDelta+2, gridOffset, gridOffset+1, gridOffset+2 );
+  const size_t numArgs = sscanf( arg, "%6d,%6d,%6d:%15f,%15f,%15f:%15f,%15f,%15f", gridDims, gridDims+1, gridDims+2, gridDelta, gridDelta+1, gridDelta+2, gridOffset, gridOffset+1, gridOffset+2 );
   if ( (numArgs != 6) && (numArgs != 9) )
     {
-    cmtk::StdErr.printf( "ERROR: target volume definition must be int,int,int:float,float,float or int,int,int:float,float,float:float,float,float\n", arg );
+    cmtk::StdErr << "ERROR: target volume definition must be int,int,int:float,float,float or int,int,int:float,float,float:float,float,float\n";
     throw cmtk::ExitException( 1 );
     }
   
@@ -112,6 +112,9 @@ cmtk::ScalarDataType DataType = cmtk::TYPE_NONE;
 cmtk::Types::DataItem OutPaddingValue = 0;
 bool OutPaddingValueFlag = true;
 
+cmtk::Types::DataItem FltPaddingValue = 0;
+bool FltPaddingValueFlag = false;
+
 const char* OutputImageName = "reformat.nii";
 
 bool CropImages = false;
@@ -121,7 +124,7 @@ void
 CallbackCropImages( const char* arg )
 {
   int cropFrom[3], cropTo[3];
-  CropImages = (6 == sscanf( arg, "%d,%d,%d,%d,%d,%d", cropFrom, cropFrom+1, cropFrom+2, cropTo,cropTo+1,cropTo+2 ) );
+  CropImages = (6 == sscanf( arg, "%6d,%6d,%6d,%6d,%6d,%6d", cropFrom, cropFrom+1, cropFrom+2, cropTo,cropTo+1,cropTo+2 ) );
 
   if ( CropImages )
     {
@@ -137,7 +140,7 @@ void
 CallbackTargetImageOffset( const char* arg )
 {
   float x, y, z;
-  if ( 3 != sscanf( arg, "%f,%f,%f", &x, &y, &z ) )
+  if ( 3 != sscanf( arg, "%15f,%15f,%15f", &x, &y, &z ) )
     {
     x = y = z = 0;
     }
@@ -154,7 +157,7 @@ void
 CallbackTargetImageOffsetPixels( const char* arg )
 {
   float x, y, z;
-  if ( 3 != sscanf( arg, "%f,%f,%f", &x, &y, &z ) )
+  if ( 3 != sscanf( arg, "%15f,%15f,%15f", &x, &y, &z ) )
     {
     x = y = z = 0;
     }
@@ -226,12 +229,12 @@ ReformatPullback()
       targetVolume = cmtk::UniformVolume::SmartPtr( cmtk::VolumeIO::ReadOriented( TargetVolumeName ) );
     else
       targetVolume = cmtk::UniformVolume::SmartPtr( cmtk::VolumeIO::ReadGridOriented( TargetVolumeName ) );
-    }
 
-  if ( ! targetVolume ) 
-    {
-    cmtk::StdErr << "ERROR: could not read target volume " << TargetVolumeName << "\n";
-    throw cmtk::ExitException( 1 );
+    if ( ! targetVolume ) 
+      {
+      cmtk::StdErr << "ERROR: could not read target volume " << TargetVolumeName << "\n";
+      throw cmtk::ExitException( 1 );
+      }
     }
 
   if ( CropImages )
@@ -260,27 +263,31 @@ ReformatPullback()
     }
   
   cmtk::UniformVolume::SmartPtr floatingVolume;
+  cmtk::TypedArray::SmartPtr floatingData( NULL );
   if ( FloatingVolumeName )
     {
     floatingVolume = cmtk::UniformVolume::SmartPtr( cmtk::VolumeIO::ReadOriented( FloatingVolumeName ) );
-    if ( ! floatingVolume ) 
+    if ( floatingVolume ) 
+      {
+      floatingData = floatingVolume->GetData();
+      if ( ! floatingData ) 
+	{
+	cmtk::StdErr << "ERROR: floating volume " << FloatingVolumeName << " seems to have no data\n";
+	throw cmtk::ExitException( 1 );
+	}
+      else
+	{
+	if ( FltPaddingValueFlag )
+	  floatingData->SetPaddingValue( FltPaddingValue );
+	}
+      }
+    else
       {
       cmtk::StdErr << "ERROR: floating volume " << FloatingVolumeName << " could not be read\n";
       throw cmtk::ExitException( 1 );
       }
     }
-  
-  cmtk::TypedArray::SmartPtr floatingData( NULL );
-  if ( floatingVolume ) 
-    {
-    floatingData = floatingVolume->GetData();
-    if ( ! floatingData ) 
-      {
-      cmtk::StdErr << "ERROR: floating volume " << FloatingVolumeName << " seems to have no data\n";
-      throw cmtk::ExitException( 1 );
-      }
-    }
-  
+
   if ( !TargetMask ) 
     {
     // unless we're in mask mode, remove target pixel data.
@@ -400,8 +407,15 @@ ReformatPullback()
       {
       if ( TargetVolumeName )
 	{
-	cmtk::ImageXformDB db( updateDB );
-	db.AddImage( OutputImageName, TargetVolumeName );
+	try 
+	  {
+	  cmtk::ImageXformDB db( updateDB );
+	  db.AddImage( OutputImageName, TargetVolumeName );
+	  }
+	catch ( const cmtk::SQLite::Exception& ex )
+	  {
+	  cmtk::StdErr << "ERROR: updating SQLite database failed - " << ex.what() <<  "\n";
+	  }	
 	}
       }
 #endif
@@ -422,7 +436,8 @@ doMain( const int argc, const char* argv[] )
     cl.SetProgramInfo( cmtk::CommandLine::PRG_DESCR, "Extended volume reformatter tool to compute reformatted images and Jacobian maps from arbitrary sequences of concatenated transformations" );
     cl.SetProgramInfo( cmtk::CommandLine::PRG_SYNTX, "reformatx [options] --floating floatingImg target x0 [x1 ...]\n  OR\n"
 		       "reformatx [options] target [x0 x1 ...] {-j,--jacobian} xx0 [xx1 ...]\n  OR\n"
-		       "WHERE x0 ... xN and xx0 ... xxN is [{-i,--inverse}] transformation##" );
+		       "WHERE x0 ... xN and xx0 ... xxN is [{-i,--inverse}] transformation##. "
+		       "(If the first transformation in the sequence is inverted, then '--inverse' must be preceded by '--', i.e., use '-- --inverse xform.path')." );
 
     typedef cmtk::CommandLine::Key Key;
     cl.BeginGroup( "PlainOptions", "Options for Plain Reformatting" );
@@ -451,6 +466,7 @@ doMain( const int argc, const char* argv[] )
     cl.AddCallback( Key( 'O', "target-offset" ), CallbackTargetImageOffset, "Override target image offset and set to x,y,z mm" );
     cl.AddCallback( Key( "target-offset-pixels" ), CallbackTargetImageOffsetPixels, "Override target image offset and set to dx,dy,dz pixels" );
     cl.AddOption( Key( 'F', "floating" ), &FloatingVolumeName, "Format and path of floating image." );
+    cl.AddOption( Key( "pad-floating" ), &FltPaddingValue, "Padding value for the floating image", &FltPaddingValueFlag );
 
     cl.BeginGroup( "Output", "Output Options" );
     cl.AddOption( Key( 'o', "outfile" ), &OutputImageName, "Format and path of output image." );
