@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4331 $
+//  $Revision: 4834 $
 //
-//  $LastChangedDate: 2012-05-09 15:02:38 -0700 (Wed, 09 May 2012) $
+//  $LastChangedDate: 2013-09-12 15:15:51 -0700 (Thu, 12 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -93,7 +93,7 @@ AddVolumeStudyVector
   if ( ! referenceVolume )
     {
     const std::string actualPath = cmtk::StrReplaceByRules( studylist.GetReferenceStudyPath(), ReplaceMap );
-    referenceVolume = cmtk::UniformVolume::SmartPtr( cmtk::VolumeIO::ReadGridOriented( actualPath.c_str() ) );
+    referenceVolume = cmtk::UniformVolume::SmartPtr( cmtk::VolumeIO::ReadGridOriented( actualPath ) );
     if ( ! referenceVolume )
       {
       cmtk::StdErr << "WARNING: could not read reference volume " << actualPath.c_str() << "\n";
@@ -102,7 +102,7 @@ AddVolumeStudyVector
     }
 
   const std::string actualPath = cmtk::StrReplaceByRules( studylist.GetFloatingStudyPath(), ReplaceMap );
-  cmtk::UniformVolume::SmartPtr floatingVolume( cmtk::VolumeIO::ReadOriented( actualPath.c_str() ) );
+  cmtk::UniformVolume::SmartPtr floatingVolume( cmtk::VolumeIO::ReadOriented( actualPath ) );
 
   if ( PaddingFlag )
     {
