@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2011 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4375 $
+//  $Revision: 4869 $
 //
-//  $LastChangedDate: 2012-05-30 13:01:18 -0700 (Wed, 30 May 2012) $
+//  $LastChangedDate: 2013-09-24 10:42:40 -0700 (Tue, 24 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -105,7 +105,7 @@ doMain( const int argc, const char* argv[] )
   
   const size_t totalNumberOfValues = seq.GetNValues() + countOverThreshold;
   printf( "Number of Values:\t%d\n", (int)totalNumberOfValues );
-  printf( "Values over Threshold:\t%d (%.2f%%)\n", countOverThreshold, 100.0 * countOverThreshold / totalNumberOfValues );
+  printf( "Values over Threshold:\t%u (%.2f%%)\n", countOverThreshold, 100.0 * countOverThreshold / totalNumberOfValues );
 
   char format[120];
   snprintf( format, sizeof( format ), "\nSTAT\tMin\tMax\tMean\tStdDev\nSTATval\t%s\t%s\t%s\t%s\n", OutputFormat, OutputFormat, OutputFormat, OutputFormat );
