@@ -2,7 +2,9 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
+//
+//  Copyright 2014 Greg Jefferis
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -51,6 +53,7 @@
 int
 doMain( const int argc, const char* argv[] )
 {
+  bool affineOnlyGlobal = false;
   cmtk::Types::Coordinate inversionTolerance = 1e-8;
 
   std::vector<std::string> inputXformPaths;
@@ -70,6 +73,7 @@ doMain( const int argc, const char* argv[] )
 
     typedef cmtk::CommandLine::Key Key;
     cl.BeginGroup( "Xform", "Transformation Options" );
+    cl.AddSwitch( Key( "affine-only" ), &affineOnlyGlobal, true, "Apply only the affine component of each transformation (or its inverse, if specified) in the given series, even if the actual transformation is nonrigid." );
     cl.AddOption( Key( "inversion-tolerance" ), &inversionTolerance, "Numerical tolerance of B-spline inversion in mm. Smaller values will lead to more accurate inversion, but may increase failure rate." );
     cl.EndGroup();
 
@@ -83,11 +87,12 @@ doMain( const int argc, const char* argv[] )
     cl.AddOption( Key( "target-image" ), &targetImagePath, "Set target image of the transformation (i.e., the image that the transformation maps points TO) to correct for differences in orientation and coordinate space." );
     cl.EndGroup();
 
-    cl.AddParameterVector( &inputXformPaths, "XformList", "List of concatenated transformations. Insert '--inverse' to use the inverse of the transformation listed next." )->SetProperties( cmtk::CommandLine::PROPS_XFORM );  
+    cl.AddParameterVector( &inputXformPaths, "XformList", "List of concatenated transformations. Insert '--inverse' to use the inverse of the transformation listed next. "
+			   "(If the first transformation in the sequence is inverted, then '--inverse' must be preceded by '--', i.e., use '-- --inverse xform.path')." )->SetProperties( cmtk::CommandLine::PROPS_XFORM );  
 
     cl.Parse( argc, argv );
     }
-  catch ( cmtk::CommandLine::Exception ex )
+  catch ( cmtk::CommandLine::Exception& ex )
     {
     cmtk::StdErr << ex << "\n";
     throw cmtk::ExitException( 1 );
@@ -95,6 +100,10 @@ doMain( const int argc, const char* argv[] )
 
   cmtk::XformList xformList = cmtk::XformListIO::MakeFromStringList( inputXformPaths );
   xformList.SetEpsilon( inversionTolerance );
+  if ( affineOnlyGlobal )
+    {
+    xformList = xformList.MakeAllAffine();
+    }
 
   if ( sourceImagePath )
     {
@@ -104,7 +113,15 @@ doMain( const int argc, const char* argv[] )
       cmtk::StdErr << "ERROR: could not read source image '" << sourceImagePath << "'\n";
       throw cmtk::ExitException( 1 );
       }
-    xformList.AddToFront( cmtk::AffineXform::SmartPtr( new cmtk::AffineXform( sourceImage->GetImageToPhysicalMatrix() ) )->GetInverse() );
+    try
+      {
+      xformList.AddToFront( cmtk::AffineXform::SmartPtr( new cmtk::AffineXform( sourceImage->GetImageToPhysicalMatrix() ) )->GetInverse() );
+      }
+    catch ( const cmtk::AffineXform::MatrixType::SingularMatrixException& )
+      {
+      cmtk::StdErr << "ERROR: singular source image-to-physical space matrix.\n";
+      throw cmtk::ExitException( 1 );
+      }
     }
   
   if ( targetImagePath )
@@ -115,10 +132,18 @@ doMain( const int argc, const char* argv[] )
       cmtk::StdErr << "ERROR: could not read target image '" << targetImagePath << "'\n";
       throw cmtk::ExitException( 1 );
       }
-    xformList.Add( cmtk::AffineXform::SmartPtr( new cmtk::AffineXform( targetImage->GetImageToPhysicalMatrix() ) ) );
+    try
+      {
+      xformList.Add( cmtk::AffineXform::SmartPtr( new cmtk::AffineXform( targetImage->GetImageToPhysicalMatrix() ) ) );
+      }
+    catch ( const cmtk::AffineXform::MatrixType::SingularMatrixException& )
+      {
+      cmtk::StdErr << "ERROR: singular target image-to-physical space matrix.\n";
+      throw cmtk::ExitException( 1 );
+      }      
     }
 
-  std::cout << std::setprecision( precision );
+  const std::streamsize originalPrecision = std::cout.precision( precision );
   
   cmtk::Xform::SpaceVectorType xyz;
   std::string restOfLine;
@@ -142,6 +167,8 @@ doMain( const int argc, const char* argv[] )
       std::cout << separator << restOfLine << std::endl;
       }
     }
+
+  std::cout.precision( originalPrecision );
   
   // if we got here, the program probably ran
   return 0;
