@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2014 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4427 $
+//  $Revision: 5129 $
 //
-//  $LastChangedDate: 2012-06-12 11:23:22 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2014-01-09 13:48:15 -0800 (Thu, 09 Jan 2014) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -52,17 +52,20 @@
 int
 doMain( const int argc, const char* argv[] )
 {
-  const char* inputPhantomPath = NULL;
-  const char* inputImagePath = NULL;
+  std::string inputPhantomPath;
+  std::string inputImagePath;
 
   cmtk::Types::Coordinate residualThreshold = 5.0;
 
-  const char* gridDims = NULL;
+  bool fitInverse = false;
+  std::string gridDims;
   cmtk::Types::Coordinate gridSpacing = 0;
-  int levels = 1;  
+
+  cmtk::FitSplineWarpToLandmarks::Parameters fittingParameters;
+  
   bool affineFirst = true;
 
-  const char* outputXform = NULL;
+  std::string outputXform;
 
   try
     {
@@ -73,7 +76,12 @@ doMain( const int argc, const char* argv[] )
 
     typedef cmtk::CommandLine::Key Key;    
     cl.BeginGroup( "Fitting", "Fitting Options" );
-    cl.AddOption( Key( "levels" ), &levels, "Number of levels in the multi-level B-spline approximation procedure." );
+    cl.AddSwitch( Key( "fit-inverse" ), &fitInverse, true, "Fit inverse transformation - this is useful for computing a Jacobian volume correction map (using 'reformatx') without having to numerically invert the fitted unwarping "
+		  "transformation." );
+    cl.AddOption( Key( "levels" ), &fittingParameters.m_Levels, "Number of levels in the multi-level B-spline approximation procedure." );
+    cl.AddOption( Key( "iterations-per-level" ), &fittingParameters.m_IterationsPerLevel, "Maximum number of spline coefficient update iterations per level in the multi-level B-spline approximation procedure." );
+    cl.AddOption( Key( "rms-threshold" ), &fittingParameters.m_ResidualThreshold, "Threshold for relative improvement of the RMS fitting residual. "
+		  "The fitting iteration terminates if (rmsAfterUpdate-rmsBeforeUpdate)/rmsBeforeUpdate < threshold." );
     cl.AddSwitch( Key( "no-fit-affine" ), &affineFirst, false, "Disable fitting of affine transformation to initialize spline. Instead, fit spline directly. This usually gives worse results and is discouraged." );
     cl.EndGroup();
 
@@ -99,7 +107,7 @@ doMain( const int argc, const char* argv[] )
     }
 
   // check for inconsistent parameters
-  if ( gridDims && gridSpacing )
+  if ( gridSpacing && !gridDims.empty() )
     {
     cmtk::StdErr << "ERROR: must specify either output spline control point spacing or grid dimensions, but not both.\n";
     throw cmtk::ExitException( 1 );
@@ -110,7 +118,15 @@ doMain( const int argc, const char* argv[] )
   cmtk::DebugOutput( 5 ) << "INFO: read phantom with " << phantom->LandmarkPairsList().size() << " landmarks.\n";  
 
   cmtk::UniformVolume::SmartConstPtr unwarpImage = cmtk::VolumeIO::ReadOriented( inputImagePath );
-  phantom->ApplyXformToLandmarks( cmtk::AffineXform( unwarpImage->GetImageToPhysicalMatrix().GetInverse() ) );
+  try
+    {
+    phantom->ApplyXformToLandmarks( cmtk::AffineXform( unwarpImage->GetImageToPhysicalMatrix().GetInverse() ) );
+    }
+  catch ( const cmtk::AffineXform::MatrixType::SingularMatrixException& )
+    {
+    cmtk::StdErr << "ERROR: singular image-to-physical space matrix cannot be inverted.\n";
+    throw cmtk::ExitException( 1 );
+    }
 
   cmtk::LandmarkPairList pairList;
   for ( std::list<cmtk::LandmarkPair>::const_iterator it = phantom->LandmarkPairsList().begin(); it != phantom->LandmarkPairsList().end(); ++it )
@@ -119,7 +135,14 @@ doMain( const int argc, const char* argv[] )
       {
       if ( it->m_Residual < residualThreshold ) // exclude outliers based on residual
 	{
-	pairList.push_back( *it );
+	if ( fitInverse )
+	  {
+	  pairList.push_back( it->GetSwapSourceTarget() );
+	  }
+	else
+	  {
+	  pairList.push_back( *it );
+	  }
 	}
       }
     }
@@ -131,27 +154,35 @@ doMain( const int argc, const char* argv[] )
   cmtk::AffineXform::SmartConstPtr affineXform;
   if ( affineFirst )
     {
-    affineXform = cmtk::FitAffineToLandmarks( pairList ).GetAffineXform();
+    try
+      {
+      affineXform = cmtk::FitAffineToLandmarks( pairList ).GetAffineXform();
+      }
+    catch ( const cmtk::AffineXform::MatrixType::SingularMatrixException& )
+      {
+      cmtk::StdErr << "ERROR: fitted affine transformation has singular matrix\n";
+      throw cmtk::ExitException( 1 );
+      }
     }
 
   // fit by final spacing
   if ( gridSpacing )
     {
-    splineWarp = cmtk::FitSplineWarpToLandmarks( pairList ).Fit( unwarpImage->m_Size, gridSpacing, levels, affineXform.GetPtr() );
+    splineWarp = cmtk::FitSplineWarpToLandmarks( pairList ).Fit( unwarpImage->m_Size, gridSpacing, affineXform.GetPtr(), fittingParameters );
     }
   else
     {
     // or fit by final control point grid dimension
-    if ( gridDims )
+    if ( !gridDims.empty() )
       {
       double dims[3];
-      if ( 3 != sscanf( gridDims, "%lf,%lf,%lf", &(dims[0]), &(dims[1]), &(dims[2]) ) )
+      if ( 3 != sscanf( gridDims.c_str(), "%20lf,%20lf,%20lf", &(dims[0]), &(dims[1]), &(dims[2]) ) )
 	{
 	cmtk::StdErr << "ERROR: grid dimensions must be specified as dimsX,dimsY,dimsZ\n";
 	throw cmtk::ExitException( 1 );
 	}
       
-      splineWarp = cmtk::FitSplineWarpToLandmarks( pairList ).Fit( unwarpImage->m_Size, cmtk::FixedVector<3,double>::FromPointer( dims ), levels, affineXform.GetPtr() );
+      splineWarp = cmtk::FitSplineWarpToLandmarks( pairList ).Fit( unwarpImage->m_Size, cmtk::FixedVector<3,double>::FromPointer( dims ), affineXform.GetPtr(), fittingParameters );
       }
     else
       {
