@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2011 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4430 $
+//  $Revision: 4870 $
 //
-//  $LastChangedDate: 2012-06-12 13:55:02 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2013-09-24 11:01:11 -0700 (Tue, 24 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -58,12 +58,12 @@
 bool Mask = false;
 bool OutputAbsolute = false;
 
-const char* RefFileName = NULL;
-const char *OutFileName = NULL;
+std::string RefFileName;
+std::string OutFileName;
 
 std::vector<std::string> InputXformPaths;
 
-const char* Downsample = NULL;
+std::string Downsample;
 
 cmtk::Types::Coordinate InversionToleranceFactor = 0.1;
 
@@ -90,7 +90,8 @@ doMain ( const int argc, const char *argv[] )
 
     cl.AddParameter( &OutFileName, "OutputPath", "Path for the output deformation field." )->SetProperties( cmtk::CommandLine::PROPS_IMAGE | cmtk::CommandLine::PROPS_OUTPUT );
     cl.AddParameter( &RefFileName, "ReferenceImage", "Input reference grid path. The dimensions and pixel size of this image determine the geometry of the output." )->SetProperties( cmtk::CommandLine::PROPS_IMAGE );
-    cl.AddParameterVector( &InputXformPaths, "XformList", "List of concatenated transformations. Insert '--inverse' to use the inverse of the transformation listed next." )->SetProperties( cmtk::CommandLine::PROPS_XFORM );  
+    cl.AddParameterVector( &InputXformPaths, "XformList", "List of concatenated transformations. Insert '--inverse' to use the inverse of the transformation listed next. "
+			   "(If the first transformation in the sequence is inverted, then '--inverse' must be preceded by '--', i.e., use '-- --inverse xform.path')." )->SetProperties( cmtk::CommandLine::PROPS_XFORM );  
  
     cl.Parse( argc, argv );
     }
@@ -115,10 +116,10 @@ doMain ( const int argc, const char *argv[] )
   
   const cmtk::XformList& xformListRef = xformList; // need this to work around GCD bug
 
-  if ( Downsample )
+  if ( !Downsample.empty() )
     {
     int factors[3] = { 1, 1, 1 };
-    const size_t nFactors = sscanf( Downsample, "%d,%d,%d", factors, factors+1, factors+2 );
+    const size_t nFactors = sscanf( Downsample.c_str(), "%6d,%6d,%6d", factors, factors+1, factors+2 );
     if ( nFactors == 1 )
       {
       factors[1] = factors[2] = factors[0];
