@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2014 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4537 $
+//  $Revision: 5129 $
 //
-//  $LastChangedDate: 2012-10-02 14:34:07 -0700 (Tue, 02 Oct 2012) $
+//  $LastChangedDate: 2014-01-09 13:48:15 -0800 (Thu, 09 Jan 2014) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -36,6 +36,7 @@
 #include <Base/cmtkUniformVolume.h>
 
 #include <System/cmtkConsole.h>
+#include <System/cmtkExitException.h>
 
 #include <algorithm>
 
@@ -53,7 +54,15 @@ AffineXform::MakeIdentityXform ()
   Types::Coordinate* scales = this->RetScales();
   if ( ! m_LogScaleFactors )
     scales[0] = scales[1] = scales[2] = 1;
-  this->ComposeMatrix();
+  try
+    {
+    this->ComposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: identify matrix construction created singular matrix - this should not happen! Aborting...\n";
+    throw ExitException( 1 );
+    }
 }
 
 AffineXform::AffineXform ( const AffineXform& other ) :
@@ -64,7 +73,15 @@ AffineXform::AffineXform ( const AffineXform& other ) :
   (*this->m_ParameterVector) = (*other.m_ParameterVector);
   this->m_LogScaleFactors = other.m_LogScaleFactors;
   this->NumberDOFs = other.NumberDOFs;
-  this->ComposeMatrix();
+  try
+    {
+    this->ComposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: construction from existing AffineXform object created singular matrix - this should not happen! Aborting...\n";
+    throw ExitException( 1 );
+    }
 }
 
 AffineXform::AffineXform
@@ -103,7 +120,16 @@ AffineXform::operator=( const AffineXform& other )
   (*this->m_ParameterVector) = (*other.m_ParameterVector);
   this->NumberDOFs = other.NumberDOFs;
   this->m_LogScaleFactors = other.m_LogScaleFactors;
-  this->ComposeMatrix();
+  try
+    {
+    this->ComposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: assignment from existing AffineXform object created singular matrix - this should not happen! Aborting...\n";
+    throw ExitException( 1 );
+    }
+
   return *this;
 }
 
@@ -147,7 +173,16 @@ AffineXform::ComposeMatrix ()
     
   // Now build matrix.
   this->Matrix.Compose( this->m_Parameters, this->m_LogScaleFactors );
-  this->UpdateInverse();
+
+  try
+    {
+    this->UpdateInverse();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: AffineXform::ComposeMatrix created singular matrix\n";
+    throw ExitException( 1 );
+    }
 }
 
 bool
@@ -289,7 +324,15 @@ void
 AffineXform::Concat( const Self& other )
 {
   this->Matrix *= other.Matrix;
-  this->DecomposeMatrix();
+  try
+    {
+    this->DecomposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: singular matrix in cmtk::AffineXform::Concat; this should not happen.\n";
+    throw ExitException( 1 );
+    }
 }
 
 void
@@ -298,7 +341,15 @@ AffineXform::Insert( const Self& other )
   Self::MatrixType composed( other.Matrix );
   composed *= this->Matrix;
   this->Matrix = composed;
-  this->DecomposeMatrix();
+  try
+    {
+    this->DecomposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: singular matrix in cmtk::AffineXform::Insert; this should not happen.\n";
+    throw ExitException( 1 );
+    }
 }
 
 void 
@@ -379,7 +430,16 @@ AffineXform::RotateWXYZ
 
   xlate = xlate.GetInverse();
   this->Matrix *= xlate;
-  this->DecomposeMatrix();
+
+  try
+    {
+    this->DecomposeMatrix();
+    }
+  catch ( const Self::MatrixType::SingularMatrixException& )
+    {
+    StdErr << "ERROR: singular matrix in cmtk::AffineXform::WXYZ; this should not happen.\n";
+    throw ExitException( 1 );
+    }
 
   if ( accumulate ) 
     {
