@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3707 $
+//  $Revision: 4812 $
 //
-//  $LastChangedDate: 2012-01-10 11:36:05 -0800 (Tue, 10 Jan 2012) $
+//  $LastChangedDate: 2013-09-09 14:26:32 -0700 (Mon, 09 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -70,11 +70,11 @@ public:
       case 2:
 	return 0.5 * xcube - 0.5 * xsquare;
       default:
-	StdErr << "weight=" << weight << " shouldn't happen!\n";
-	throw ExitException( 1 );
 	break;
       }
-    return 0;
+
+    StdErr << "weight=" << weight << " shouldn't happen!\n";
+    throw ExitException( 1 );
   }
 };
 
