@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4562 $
+//  $Revision: 5083 $
 //
-//  $LastChangedDate: 2012-10-23 11:29:15 -0700 (Tue, 23 Oct 2012) $
+//  $LastChangedDate: 2013-12-03 15:22:05 -0800 (Tue, 03 Dec 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -90,6 +90,17 @@ DataGrid::GetWholeImageRegion() const
   return Self::RegionType( Self::IndexType( 0 ), Self::IndexType( this->m_Dims ) );
 }
 
+const DataGrid::RegionType
+DataGrid::GetSliceRegion( const int axis, const int slice ) const
+{
+  Self::IndexType regionFrom( 0 ), regionTo( this->m_Dims );
+
+  regionFrom[axis] = slice;
+  regionTo[axis] = slice+1;
+
+  return Self::RegionType( regionFrom, regionTo );
+}
+
 TypedArray::SmartPtr
 DataGrid::CreateDataArray( const ScalarDataType dataType, const bool setToZero )
 {
@@ -389,7 +400,7 @@ DataGrid::MirrorPlaneInPlace
     }
 }
 
-ScalarImage*
+ScalarImage::SmartPtr
 DataGrid::GetOrthoSlice
 ( const int axis, const unsigned int plane ) const
 {
@@ -456,14 +467,13 @@ DataGrid::GetOrthoSlice
       }
     }
   
-  ScalarImage* sliceImage = new ScalarImage( dims[0], dims[1] );
-  sliceImage->SetPixelData( TypedArray::SmartPtr( sliceData ) );
-  
+  ScalarImage::SmartPtr sliceImage( new ScalarImage( dims[0], dims[1] ) );
+  sliceImage->SetPixelData( sliceData );
   return sliceImage;
 }
 
-DataGrid*
-DataGrid::ExtractSliceVirtual
+DataGrid::SmartPtr
+DataGrid::ExtractSlice
 ( const int axis, const int plane ) const
 {
   unsigned int dims[2], incX, incY, incZ;
@@ -528,7 +538,7 @@ DataGrid::ExtractSliceVirtual
   IndexType newDims = this->m_Dims;
   newDims[axis] = 1;
 
-  return new DataGrid( newDims, sliceData );
+  return Self::SmartPtr( new Self( newDims, sliceData ) );
 }
 
 void 
