@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4426 $
+//  $Revision: 5083 $
 //
-//  $LastChangedDate: 2012-06-12 10:30:56 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2013-12-03 15:22:05 -0800 (Tue, 03 Dec 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -286,6 +286,9 @@ public:
   /// Get whole image region.
   const Self::RegionType GetWholeImageRegion() const;
 
+  /// Get region covering one slice of the image.
+  const Self::RegionType GetSliceRegion( const int axis /*!< Coordinate axis perpendicular to the slice, i.e., 0 for x, 1 for y, 2 for z.*/, const int slice /*!< Index of selected slice. */ ) const;
+
   /// Get index increments for crop region.
   const Self::IndexType GetCropRegionIncrements() const;
 
@@ -322,15 +325,12 @@ public:
   
   /** Return orthogonal slice as a 2D image.
    */
-  virtual ScalarImage* GetOrthoSlice( const int axis, const unsigned int plane ) const;
+  virtual ScalarImage::SmartPtr GetOrthoSlice( const int axis, const unsigned int plane ) const;
   
   /** Extract orthogonal slice as a data grid object.
    */
-  Self::SmartPtr ExtractSlice( const int axis, const int plane ) const
-  {
-    return Self::SmartPtr( this->ExtractSliceVirtual( axis, plane ) );
-  }
-  
+  Self::SmartPtr ExtractSlice( const int axis /*!< Coordinate axis perpendicular to extracted plane*/, const int plane /*!< Index of extracted plane */ ) const;
+
   /** Set orthogonal slice from a 2D image.
    */
   virtual void SetOrthoSlice( const int axis, const unsigned int idx, const ScalarImage* slice );
@@ -419,10 +419,6 @@ protected:
   /// Offset to next column, row, and plane.
   int nextIJK;
 
-  /** Extract orthogonal slice as a data grid object.
-   */
-  virtual Self* ExtractSliceVirtual( const int axis /*!< Coordinate axis perpendicular to extracted plane*/, const int plane /*!< Index of extracted plane */ ) const;
-
 private:
   /** Crop region.
    */
