@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,18 +22,19 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4278 $
+//  $Revision: 5000 $
 //
-//  $LastChangedDate: 2012-04-30 10:53:33 -0700 (Mon, 30 Apr 2012) $
+//  $LastChangedDate: 2013-10-21 12:11:07 -0700 (Mon, 21 Oct 2013) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
 #include "cmtkDataGridMorphologicalOperators.h"
 
 #include <Base/cmtkTemplateArray.h>
-#include <System/cmtkException.h>
+
+#include <System/cmtkExitException.h>
 
 #include <vector>
 
@@ -44,89 +45,19 @@ cmtk
 DataGridMorphologicalOperators
 ::DataGridMorphologicalOperators( const DataGrid::SmartConstPtr& dataGrid ) 
   : m_DataGrid( dataGrid ) 
-{
-  if ( !this->m_DataGrid->GetData() )
-    throw Exception( "ERROR: DataGrid object given to DataGridMorphologicalOperators constructor does not have a data array" );
-}
-
-bool
-DataGridMorphologicalOperators::EliminatePaddingVoting( const int iterations )
-{
-  bool changed = false;
-  for ( int it = 0; (it < iterations) && changed; ++it )
-    {
-    const TypedArray* data = this->m_DataGrid->GetData();
-    TypedArray::SmartPtr result = data->Clone();
-    
-    if ( !data->GetPaddingFlag() ) return changed;
-    const Types::DataItem dataPadding = data->GetPaddingValue();
-    
-    short valueMap[256];  
-    size_t offset = 0;
-    for ( int z = 0; z < this->m_DataGrid->m_Dims[2]; ++z ) 
-      {
-      const int dzFrom = z ? z-1 : z, dzTo = (z<this->m_DataGrid->m_Dims[2]-1) ? z+1 : z;
-      for ( int y = 0; y < this->m_DataGrid->m_Dims[1]; ++y ) 
-	{
-	const int dyFrom = y ? y-1 : y, dyTo = (y<this->m_DataGrid->m_Dims[1]-1) ? y+1 : y;
-	for ( int x = 0; x < this->m_DataGrid->m_Dims[0]; ++x, ++offset ) 
-	  {
-	  // is there Padding here?
-	  if ( data->PaddingDataAt( offset ) )
-	    {
-	    // determine frequencies of values among neighbors
-	    memset( valueMap, 0, sizeof( valueMap ) );
-	    const int dxFrom = x ? x-1 : x, dxTo = (x<this->m_DataGrid->m_Dims[0]-1) ? x+1 : x;
-	    for ( int dz = dzFrom; (dz <= dzTo); ++dz )
-	      for ( int dy = dyFrom; (dy <= dyTo); ++dy )
-		for ( int dx = dxFrom; (dx <= dxTo); ++dx )
-		  {
-		  Types::DataItem value;
-		  if ( this->m_DataGrid->GetDataAt( value, dx, dy, dz ) )
-		    {
-		    ++valueMap[static_cast<byte>( value )];
-		    }
-		  }
-	    
-	    // find most frequent value among neighbors
-	    char maxCount = 0;
-	    Types::DataItem maxValue = 0;
-	    
-	    for ( int it = 0; it < 256; ++it )
-	      {
-	      if ( valueMap[it] > maxCount )
-		{
-		maxCount = static_cast<char>( valueMap[it] );
-		maxValue = it;
-		}
-	      else
-		if ( valueMap[it] == maxCount )
-		  {
-		  maxValue = dataPadding;
-		  }
-	      }
-	    
-	    if ( maxValue != dataPadding )
-	      {
-	      changed = true;
-	      result->Set( maxValue, offset );
-	      }
-	    } // if PaddingDataAt()
-	  } // for x
-	} // for y
-      } // for z
-    }
-  
-  return changed;
-}
+{}
 
 TypedArray::SmartPtr
 DataGridMorphologicalOperators::GetEroded( const int iterations ) const
 {
   TypedArray::SmartPtr dataArray = this->m_DataGrid->GetData();
+  if ( ! dataArray )
+    return TypedArray::SmartPtr( NULL );
+
+  // if not byte array, convert first (copy will be de-allocated automatically upon exiting this function).
   if ( dataArray->GetType() != TYPE_BYTE ) 
     {
-    throw Exception( "ERROR: convert data to byte before calling DataGridMorphologicalOperators::GetDataErode()" );
+    dataArray = dataArray->Convert( TYPE_BYTE );
     }
 
   const byte* data = static_cast<const byte*>( dataArray->GetDataPtr() );
@@ -181,9 +112,13 @@ TypedArray::SmartPtr
 DataGridMorphologicalOperators::GetDilated( const int iterations ) const
 {
   TypedArray::SmartPtr dataArray = this->m_DataGrid->GetData();
+  if ( ! dataArray )
+    return TypedArray::SmartPtr( NULL );
+
+  // if not byte array, convert first (copy will be de-allocated automatically upon exiting this function).
   if ( dataArray->GetType() != TYPE_BYTE ) 
     {
-    throw Exception( "ERROR: convert data to byte before calling DataGridMorphologicalOperators::GetDataDilate()" );
+    dataArray = dataArray->Convert( TYPE_BYTE );
     }
   
   const byte* data = static_cast<const byte*>( dataArray->GetDataPtr() );
@@ -237,6 +172,9 @@ TypedArray::SmartPtr
 DataGridMorphologicalOperators::GetBoundaryMap( const bool multiValued ) const
 {
   TypedArray::SmartPtr dataArray = this->m_DataGrid->GetData();
+  if ( ! dataArray )
+    return TypedArray::SmartPtr( NULL );
+
   ShortArray::SmartPtr boundaryArray = ShortArray::Create( dataArray->GetDataSize() );
   short* boundary = boundaryArray->GetDataPtrConcrete();
 
