@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4324 $
+//  $Revision: 4906 $
 //
-//  $LastChangedDate: 2012-05-08 11:12:50 -0700 (Tue, 08 May 2012) $
+//  $LastChangedDate: 2013-10-01 11:35:08 -0700 (Tue, 01 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -156,7 +156,7 @@ DataGrid::GetRegionData( const Self::RegionType& region ) const
 {
   const TypedArray* srcData = this->GetData();
   if ( ! srcData ) 
-    throw( Exception( "No input data in DataGrid::GetRegionData()" ) );
+    return TypedArray::SmartPtr( NULL );
 
   TypedArray::SmartPtr cropData = TypedArray::Create( srcData->GetType(), region.Size() );
   
