@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4381 $
+//  $Revision: 4538 $
 //
-//  $LastChangedDate: 2012-05-30 14:18:15 -0700 (Wed, 30 May 2012) $
+//  $LastChangedDate: 2012-10-02 15:16:13 -0700 (Tue, 02 Oct 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -112,7 +112,7 @@ public:
   void InitControlPoints( const AffineXform* affineXform = NULL );
 
   /// Apply transformation to vector in-place.
-  virtual void ApplyInPlace ( Self::SpaceVectorType& ) const;
+  virtual Self::SpaceVectorType Apply ( const Self::SpaceVectorType& ) const;
 
   /** Return origin of warped vector.
    */
@@ -122,14 +122,6 @@ public:
     return false;
   }
 
-  /** Return origin of warped vector.
-   */
-  virtual bool ApplyInverseInPlace( Self::SpaceVectorType&, const Types::Coordinate = 0.01  ) const 
-  {
-    // not implemented
-    return false;
-  }
-
   /** Get the deformed position of a transformation control point.
    *\note This function does not necessarily return the shifted control point position,
    * but rather it applies the current transformation to the given control
