@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4538 $
+//  $Revision: 4870 $
 //
-//  $LastChangedDate: 2012-10-02 15:16:13 -0700 (Tue, 02 Oct 2012) $
+//  $LastChangedDate: 2013-09-24 11:01:11 -0700 (Tue, 24 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -85,11 +85,10 @@ SplineWarpXform::GetJacobianRow
   std::vector<Types::Coordinate> phiHatZ(3*numberOfCells);
 #endif
 
-  const int *gpo;
   int phiIdx = 0;
   for ( int cell = 0; cell < numberOfCells; ++cell, coeff += nextI ) 
     {
-    gpo = &GridPointOffset[0];
+    const int *gpo = &GridPointOffset[0];
     for ( int dim = 0; dim < 3; ++dim, ++phiIdx ) 
       {
       phiCompX = phiCompY = phiCompZ = 0;
@@ -418,11 +417,10 @@ SplineWarpXform::GetJacobianDeterminantRow
   std::vector<Types::Coordinate> phiHatZ(3*numberOfCells);
 #endif
 
-  const int *gpo;
   int phiIdx = 0;
   for ( int cell = 0; cell < numberOfCells; ++cell, coeff += nextI ) 
     {
-    gpo = &GridPointOffset[0];
+    const int *gpo = &GridPointOffset[0];
     for ( int dim = 0; dim < 3; ++dim, ++phiIdx ) 
       {
       phiCompX = phiCompY = phiCompZ = 0;
@@ -786,9 +784,9 @@ SplineWarpXform::RelaxToUnfold()
 	  }
 	
 	// copy modified control point position component back
-	for ( size_t cp = 0; cp < this->m_NumberOfControlPoints; ++cp )
+	for ( size_t cp2 = 0; cp2 < this->m_NumberOfControlPoints; ++cp2 )
 	  {
-	  pureDeformation[dim+cp*3] = smoothed[cp];
+	  pureDeformation[dim+cp2*3] = smoothed[cp2];
 	  }
 	}
       
