@@ -2,7 +2,7 @@
 //
 //  Copyright 2004-2012 SRI International
 //
-//  Copyright 1997-2011 Torsten Rohlfing
+//  Copyright 1997-2011, 2013 Torsten Rohlfing
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4408 $
+//  $Revision: 4812 $
 //
-//  $LastChangedDate: 2012-06-01 14:54:11 -0700 (Fri, 01 Jun 2012) $
+//  $LastChangedDate: 2013-09-09 14:26:32 -0700 (Mon, 09 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -155,9 +155,6 @@ template<class T>
 class Range
 {
 public:
-  /// Default constructor: do nothing at all.
-  Range() {}
-
   /// Constructor.
   Range( const T& lowerBound, const T& upperBound ) : m_LowerBound( lowerBound ), m_UpperBound( upperBound ) {}
 
