@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4454 $
+//  $Revision: 5021 $
 //
-//  $LastChangedDate: 2012-07-09 15:01:19 -0700 (Mon, 09 Jul 2012) $
+//  $LastChangedDate: 2013-11-23 09:17:13 -0800 (Sat, 23 Nov 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -40,7 +40,7 @@ cmtk
 {
 
 UniformVolume::UniformVolume 
-( const UniformVolume& other ) : Volume( other ), m_Delta( other.m_Delta ), m_IndexToPhysicalMatrix( other.m_IndexToPhysicalMatrix ) {}
+( const UniformVolume& other ) : Volume( other ), m_Delta( other.m_Delta ), m_IndexToPhysicalMatrix( other.m_IndexToPhysicalMatrix ), m_AlternativeIndexToPhysicalMatrices( other.m_AlternativeIndexToPhysicalMatrices ) {}
 
 UniformVolume::UniformVolume
 ( const DataGrid::IndexType& dims, const Self::CoordinateVectorType& size, TypedArray::SmartPtr& data )
@@ -106,7 +106,7 @@ UniformVolume::GetResampled
   UniformVolume* newVolume = new UniformVolume( newDims, newSize );
   newVolume->SetData( TypedArray::SmartPtr( newVolume->Resample( *this ) ) );
   
-  newVolume->m_IndexToPhysicalMatrix = this->m_IndexToPhysicalMatrix;
+  newVolume->SetImageToPhysicalMatrix( this->GetImageToPhysicalMatrix() );
   newVolume->SetHighResCropRegion( this->GetHighResCropRegion() );
   newVolume->SetOffset( this->m_Offset );
   newVolume->CopyMetaInfo( *this );
@@ -156,6 +156,7 @@ UniformVolume::CloneGridVirtual() const
   clone->SetOffset( this->m_Offset );
   clone->CopyMetaInfo( *this );
   clone->m_IndexToPhysicalMatrix = this->m_IndexToPhysicalMatrix;
+  clone->m_AlternativeIndexToPhysicalMatrices = this->m_AlternativeIndexToPhysicalMatrices;
   return clone;
 }
 
@@ -202,12 +203,30 @@ UniformVolume::GetDownsampledAndAveraged( const int (&downsample)[3] ) const
 
   // apply offset shift to index-to-physical matrix
   for ( int axis = 0; axis < 3; ++axis )
+    {
     for ( int i = 0; i < 3; ++i )
       {
       dsVolume->m_IndexToPhysicalMatrix[3][i] += (downsample[i]-1) * dsVolume->m_IndexToPhysicalMatrix[axis][i] / 2;
       // also update voxel size
       dsVolume->m_IndexToPhysicalMatrix[axis][i] *= downsample[i];
       }
+    }
+
+  // do the same for any alternative matrices
+  dsVolume->m_AlternativeIndexToPhysicalMatrices = this->m_AlternativeIndexToPhysicalMatrices;
+  for ( std::map<int,AffineXform::MatrixType>::iterator it = dsVolume->m_AlternativeIndexToPhysicalMatrices.begin(); it != dsVolume->m_AlternativeIndexToPhysicalMatrices.end(); ++it )
+    {
+    // apply offset shift to index-to-physical matrix
+    for ( int axis = 0; axis < 3; ++axis )
+      {
+      for ( int i = 0; i < 3; ++i )
+	{
+	(it->second)[3][i] += (downsample[i]-1) * (it->second)[axis][i] / 2;
+	// also update voxel size
+	(it->second)[axis][i] *= downsample[i];
+	}
+      }
+    }
   
   return dsVolume;
 }
@@ -254,6 +273,22 @@ UniformVolume::GetDownsampled( const int (&downsample)[3] ) const
       dsVolume->m_IndexToPhysicalMatrix[axis][i] *= downsample[i];
       }
   
+  // do the same for any alternative matrices
+  dsVolume->m_AlternativeIndexToPhysicalMatrices = this->m_AlternativeIndexToPhysicalMatrices;
+  for ( std::map<int,AffineXform::MatrixType>::iterator it = dsVolume->m_AlternativeIndexToPhysicalMatrices.begin(); it != dsVolume->m_AlternativeIndexToPhysicalMatrices.end(); ++it )
+    {
+    // apply offset shift to index-to-physical matrix
+    for ( int axis = 0; axis < 3; ++axis )
+      {
+      for ( int i = 0; i < 3; ++i )
+	{
+	(it->second)[3][i] += (downsample[i]-1) * (it->second)[axis][i] / 2;
+	// also update voxel size
+	(it->second)[axis][i] *= downsample[i];
+	}
+      }
+    }
+
   return dsVolume;
 }
 
@@ -295,6 +330,18 @@ UniformVolume::GetInterleavedSubVolume
   for ( int i = 0; i < 3; ++i )
     volume->m_IndexToPhysicalMatrix[axis][i] *= factor;
 
+  // do the same for any alternative matrices
+  volume->m_AlternativeIndexToPhysicalMatrices = this->m_AlternativeIndexToPhysicalMatrices;
+  for ( std::map<int,AffineXform::MatrixType>::iterator it = volume->m_AlternativeIndexToPhysicalMatrices.begin(); it != volume->m_AlternativeIndexToPhysicalMatrices.end(); ++it )
+    {
+    // update coordinate offset according to sub-volume index
+    for ( int i = 0; i < 3; ++i )
+      (it->second)[3][i] += idx * (it->second)[axis][i];
+    // scale direction vector along axis
+    for ( int i = 0; i < 3; ++i )
+      (it->second)[axis][i] *= factor;
+    }  
+
   if ( this->GetData()->GetPaddingFlag() )
     {
     volume->GetData()->SetPaddingValue( this->GetData()->GetPaddingValue() );
@@ -323,6 +370,17 @@ UniformVolume::GetInterleavedPaddedSubVolume
   
   volume->CopyMetaInfo( *this );
   volume->m_IndexToPhysicalMatrix = this->m_IndexToPhysicalMatrix;
+  volume->m_AlternativeIndexToPhysicalMatrices = this->m_AlternativeIndexToPhysicalMatrices;
+  for ( std::map<int,AffineXform::MatrixType>::iterator it = volume->m_AlternativeIndexToPhysicalMatrices.begin(); it != volume->m_AlternativeIndexToPhysicalMatrices.end(); ++it )
+    {
+    // update coordinate offset according to sub-volume index
+    for ( int i = 0; i < 3; ++i )
+      (it->second)[3][i] += idx * (it->second)[axis][i];
+    // scale direction vector along axis
+    for ( int i = 0; i < 3; ++i )
+      (it->second)[axis][i] *= factor;
+    }
+
   return volume;
 }
 
@@ -350,11 +408,11 @@ UniformVolume::Mirror ( const int axis )
   this->CropRegion().To()[ axis ] = this->m_Dims[ axis ] - 1 - this->CropRegion().To()[ axis ];
 }
 
-ScalarImage*
+ScalarImage::SmartPtr
 UniformVolume::GetOrthoSlice
 ( const int axis, const unsigned int plane ) const
 {
-  ScalarImage* sliceImage = DataGrid::GetOrthoSlice( axis, plane );
+  ScalarImage::SmartPtr sliceImage = DataGrid::GetOrthoSlice( axis, plane );
   sliceImage->SetImageSlicePosition( this->GetPlaneCoord( axis, plane ) );
 
   Self::CoordinateVectorType imageOffset( 0.0 );
@@ -389,12 +447,12 @@ UniformVolume::GetOrthoSlice
   return sliceImage;
 }
 
-UniformVolume*
-UniformVolume::ExtractSliceVirtual
+UniformVolume::SmartPtr
+UniformVolume::ExtractSlice
 ( const int axis, const int plane ) const
 {
-  DataGrid::SmartPtr sliceGrid( this->DataGrid::ExtractSliceVirtual( axis, plane ) );
-  Self* sliceVolume = new Self( sliceGrid->m_Dims, this->m_Delta[0], this->m_Delta[1], this->m_Delta[2], sliceGrid->m_Data );
+  DataGrid::SmartPtr sliceGrid( this->DataGrid::ExtractSlice( axis, plane ) );
+  Self::SmartPtr sliceVolume( new Self( sliceGrid->m_Dims, this->m_Delta[0], this->m_Delta[1], this->m_Delta[2], sliceGrid->m_Data ) );
 
   sliceVolume->m_Offset = this->m_Offset;
   sliceVolume->m_Offset[axis] += plane * this->m_Delta[axis];
@@ -402,14 +460,14 @@ UniformVolume::ExtractSliceVirtual
   return sliceVolume;
 }
 
-ScalarImage*
+ScalarImage::SmartPtr
 UniformVolume::GetNearestOrthoSlice
 ( const int axis, const Types::Coordinate location ) const
 {
   return this->GetOrthoSlice( axis, this->GetCoordIndex( axis, location ) );
 }
 
-ScalarImage*
+ScalarImage::SmartPtr
 UniformVolume::GetOrthoSliceInterp
 ( const int axis, const Types::Coordinate location ) const
 {
@@ -426,8 +484,8 @@ UniformVolume::GetOrthoSliceInterp
   if ( ((nextSliceLocation - location) / (nextSliceLocation-baseSliceLocation )) < 0.01 )
     return this->GetOrthoSlice( axis, baseSliceIndex+1 );
 
-  ScalarImage* image0 = this->GetOrthoSlice( axis, baseSliceIndex );
-  ScalarImage* image1 = this->GetOrthoSlice( axis, baseSliceIndex+1 );
+  ScalarImage::SmartPtr image0 = this->GetOrthoSlice( axis, baseSliceIndex );
+  ScalarImage::SmartPtr image1 = this->GetOrthoSlice( axis, baseSliceIndex+1 );
 
   TypedArray::SmartPtr data0 = image0->GetPixelData();
   TypedArray::SmartPtr data1 = image1->GetPixelData();
@@ -446,11 +504,10 @@ UniformVolume::GetOrthoSliceInterp
       data0->SetPaddingAt( idx );
       }
     }
-  delete image1;
   
   image0->SetImageSlicePosition( location );
   image0->SetImageOrigin( weight0 * image0->GetImageOrigin() + (1-weight0) * image1->GetImageOrigin() );
-  
+
   return image0;
 }
 
