@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4538 $
+//  $Revision: 4882 $
 //
-//  $LastChangedDate: 2012-10-02 15:16:13 -0700 (Tue, 02 Oct 2012) $
+//  $LastChangedDate: 2013-09-27 15:16:36 -0700 (Fri, 27 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -172,12 +172,14 @@ public:
   WarpXform () : 
     m_InitialAffineXform( NULL ), 
     m_NumberOfControlPoints( 0 ),
+    m_GlobalScaling( 1.0 ),
     m_ActiveFlags( NULL )
   { 
     this->m_IgnoreEdge = 0; 
     this->m_FastMode = false; 
     this->m_Dims[0] = this->m_Dims[1] = this->m_Dims[2] = 0;
     this->m_InverseSpacing[0] = this->m_InverseSpacing[1] = this->m_InverseSpacing[2] = 0.0;
+    this->nextI = this->nextJ = this->nextK = this->nextIJ = this->nextIK = this->nextJK = this->nextIJK = 0;
   }
 
   /// Destructor.
