@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2012 SRI International
+//  Copyright 2012, 2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -20,9 +20,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4326 $
+//  $Revision: 4898 $
 //
-//  $LastChangedDate: 2012-05-08 12:05:35 -0700 (Tue, 08 May 2012) $
+//  $LastChangedDate: 2013-09-30 16:10:27 -0700 (Mon, 30 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -30,6 +30,8 @@
 
 #include "cmtkPhantomIO.h"
 
+#include <System/cmtkCoverity.h>
+
 #include <stdio.h>
 
 const char *
@@ -47,17 +49,21 @@ cmtk::PhantomIO::WhitespaceWriteMiniXML( mxml_node_t* node, int where)
 
   static const wsLookupType wsLookup[] = 
   {
-    { "phantomType",  { NULL, NULL, NULL, "\n" } },
-    { "snr",          { NULL, NULL, NULL, "\n" } },
-    { "cnr",          { NULL, NULL, NULL, "\n" } },
-    { "landmarkList", { NULL, "\n", NULL, "\n" } },
-    { "landmark",     { "\t", "\n", "\t", "\n" } },
-    { "name",         { "\t\t", NULL, NULL, "\n" } },
-    { "detected",     { "\t\t", NULL, NULL, "\n" } },
-    { "expected",     { "\t\t", NULL, NULL, "\n" } },
-    { "isPrecise",    { "\t\t", NULL, NULL, "\n" } },
-    { "fitResidual",  { "\t\t", NULL, NULL, "\n" } },
-    { NULL, {NULL, NULL, NULL, NULL} }
+    { "phantomType",             { NULL, NULL, NULL, "\n" } },
+    { "fallbackOrientationCNR",  { NULL, "\n", NULL, "\n" } },
+    { "fallbackCentroidCNR",     { NULL, "\n", NULL, "\n" } },
+    { "snr",                     { NULL, NULL, NULL, "\n" } },
+    { "cnr",                     { NULL, NULL, NULL, "\n" } },
+    { "scale",                   { NULL, NULL, NULL, "\n" } },
+    { "nonlinear",               { NULL, NULL, NULL, "\n" } },
+    { "landmarkList",            { NULL, "\n", NULL, "\n" } },
+    { "landmark",                { "\t", "\n", "\t", "\n" } },
+    { "name",                    { "\t\t", NULL, NULL, "\n" } },
+    { "detected",                { "\t\t", NULL, NULL, "\n" } },
+    { "expected",                { "\t\t", NULL, NULL, "\n" } },
+    { "isPrecise",               { "\t\t", NULL, NULL, "\n" } },
+    { "fitResidual",             { "\t\t", NULL, NULL, "\n" } },
+    { NULL,                      {NULL, NULL, NULL, NULL} }
   };
   
   if ( (where >= 0) && (where < 4) )
@@ -91,37 +97,69 @@ cmtk::PhantomIO::Write( const DetectedPhantomMagphanEMR051& phantom, const std::
   mxml_node_t *x_root = mxmlNewElement( NULL, "?xml version=\"1.0\" encoding=\"utf-8\"?" );
   
   mxml_node_t *x_phantom = mxmlNewElement( x_root, "phantom" );
-  mxmlNewText( mxmlNewElement( x_phantom, "phantomType" ), 0, "MagphanEMR051" );
-  mxmlNewReal( mxmlNewElement( x_phantom, "snr" ), phantom.m_EstimatedSNR ); 
+  Coverity::FakeFree( mxmlNewText( mxmlNewElement( x_phantom, "phantomType" ), 0, "MagphanEMR051" ) );
+
+  // put fallback flags into output as a warning
+  if ( phantom.m_StatusFlags.m_FallbackOrientationCNR )
+    {
+    Coverity::FakeFree( mxmlNewElement( x_phantom, "fallbackOrientationCNR" ) );
+    }
+  if ( phantom.m_StatusFlags.m_FallbackCentroidCNR )
+    {
+    mxml_node_t *x_fallback_centroid = mxmlNewElement( x_phantom, "fallbackCentroidCNR" );
+    char distStr[10];
+    snprintf( distStr, 10, "%8f", phantom.m_StatusFlags.m_DistanceSNRtoCNR );
+    mxmlElementSetAttr( x_fallback_centroid, "distance", distStr );
+    Coverity::FakeFree( x_fallback_centroid );
+    }
+
+  Coverity::FakeFree( mxmlNewReal( mxmlNewElement( x_phantom, "snr" ), phantom.m_EstimatedSNR ) ); 
 
   mxml_node_t *x_cnr = mxmlNewElement( x_phantom, "cnr" );
   for ( size_t i=0; i < phantom.m_EstimatedCNR.Size(); ++i )
-    mxmlNewReal( x_cnr, phantom.m_EstimatedCNR[i] ); 
+    Coverity::FakeFree( mxmlNewReal( x_cnr, phantom.m_EstimatedCNR[i] ) );
     
+  FixedVector<3,Types::Coordinate> scales = phantom.m_LinearFitXform.GetScales(); 
+  mxml_node_t *x_scale= mxmlNewElement( x_phantom, "scale");
+  for ( size_t idx = 0; idx < 3; ++idx )
+    {
+    Coverity::FakeFree( mxmlNewReal( x_scale, scales[idx] ) );
+    }
+  
+  mxml_node_t *x_nonlinear= mxmlNewElement( x_phantom, "nonlinear");
+  for ( size_t idx = 0; idx < 3; ++idx )
+    {
+    Coverity::FakeFree( mxmlNewReal( x_nonlinear, phantom.m_EstimatedNonLinear[idx] ) );
+    }
+  
   mxml_node_t *x_lmpairs = mxmlNewElement( x_phantom, "landmarkList" );
   mxmlElementSetAttr( x_lmpairs, "coordinates", "physical" );
-  mxmlElementSetAttr( x_lmpairs, "space", "RAS" );
+  mxmlElementSetAttr( x_lmpairs, "space", "RAS" );  
 
   const std::list<LandmarkPair>& lmPairs = phantom.LandmarkPairsList();
+  char lmCntStr[5];
+  snprintf( lmCntStr, 4, "%d", static_cast<byte>( lmPairs.size() ) );
+  mxmlElementSetAttr( x_lmpairs, "count", lmCntStr );    
+
   for ( std::list<LandmarkPair>::const_iterator it = lmPairs.begin(); it != lmPairs.end(); ++it )
     {
     mxml_node_t *x_lm = mxmlNewElement( x_lmpairs, "landmark");
     
-    mxmlNewText( mxmlNewElement( x_lm, "name" ), 0, it->m_Name.c_str() ); 
+    Coverity::FakeFree( mxmlNewText( mxmlNewElement( x_lm, "name" ), 0, it->m_Name.c_str() ) );
     mxml_node_t *x_expected = mxmlNewElement( x_lm, "expected");
     for ( size_t idx = 0; idx < 3; ++idx )
       {
-      mxmlNewReal( x_expected, it->m_Location[idx] );
+      Coverity::FakeFree( mxmlNewReal( x_expected, it->m_Location[idx] ) );
       }
     
     mxml_node_t *x_detected= mxmlNewElement( x_lm, "detected");
     for ( size_t idx = 0; idx < 3; ++idx )
       {
-      mxmlNewReal( x_detected, it->m_TargetLocation[idx] );
+      Coverity::FakeFree( mxmlNewReal( x_detected, it->m_TargetLocation[idx] ) );
       }
     
-    mxmlNewText( mxmlNewElement( x_lm, "isPrecise" ), 0, it->m_Precise ? "yes" : "no" );
-    mxmlNewReal( mxmlNewElement( x_lm, "fitResidual" ), it->m_Residual );    
+    Coverity::FakeFree( mxmlNewText( mxmlNewElement( x_lm, "isPrecise" ), 0, it->m_Precise ? "yes" : "no" ) );
+    Coverity::FakeFree( mxmlNewReal( mxmlNewElement( x_lm, "fitResidual" ), it->m_Residual ) );
     }
   
   FILE *file = fopen( fpath.c_str(), "w" );
