@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4346 $
+//  $Revision: 5047 $
 //
-//  $LastChangedDate: 2012-05-11 15:02:47 -0700 (Fri, 11 May 2012) $
+//  $LastChangedDate: 2013-11-27 14:59:44 -0800 (Wed, 27 Nov 2013) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -151,8 +151,7 @@ TypedStreamInput
       }
     return;
     }
-  int releaseMinor, releaseMajor;
-  if (2 != sscanf( Buffer+1, " TYPEDSTREAM %d.%d", &releaseMajor, &releaseMinor)) 
+  if (2 != sscanf( Buffer+1, " TYPEDSTREAM %4d.%4d", &this->m_ReleaseMajor, &this->m_ReleaseMinor)) 
     {
     this->m_Status = Self::ERROR_FORMAT;
     if ( GzFile )
@@ -167,6 +166,13 @@ TypedStreamInput
       File = NULL;
       }
     }
+  else
+    {
+    if ( (this->m_ReleaseMajor > CMTK_VERSION_MAJOR) || ( (this->m_ReleaseMajor == CMTK_VERSION_MAJOR) && (this->m_ReleaseMinor > CMTK_VERSION_MINOR)) )
+      {
+      StdErr << "WARNING: input archive was written by newer version of CMTK (" << this->m_ReleaseMajor << "." << this->m_ReleaseMinor << " or higher) - proceed with caution.\n";
+      }
+    }
 }
 
 void
@@ -207,9 +213,22 @@ TypedStreamInput
     }
 
   if ( GzFile )
-    gzseek( GzFile, LevelStack.top(), SEEK_SET );
+    {
+    if ( -1 == gzseek( GzFile, LevelStack.top(), SEEK_SET ) )
+      {
+      this->m_Status = Self::ERROR_SYSTEM;
+      return Self::CONDITION_ERROR;
+      }
+    }
   else
-    fseek( File, LevelStack.top(), SEEK_SET );
+    {
+    if ( fseek( File, LevelStack.top(), SEEK_SET ) )
+      {
+      this->m_Status = Self::ERROR_SYSTEM;
+      return Self::CONDITION_ERROR;
+      }
+    }
+
   return Self::CONDITION_OK;
 }
 
@@ -277,21 +296,37 @@ TypedStreamInput
       {
       if ( initialLevel ) 
 	{
-	gzseek( GzFile, LevelStack.top(), SEEK_SET );
+	if ( -1 == gzseek( GzFile, LevelStack.top(), SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
 	} 
       else 
 	{
-	gzseek( GzFile, 0, SEEK_SET);
+	if ( -1 == gzseek( GzFile, 0, SEEK_SET) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
 	}
       } 
     else
       if ( initialLevel ) 
 	{
-	fseek( File, LevelStack.top(), SEEK_SET );
+	if ( fseek( File, LevelStack.top(), SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
 	} 
       else 
 	{
-	fseek( File, 0, SEEK_SET);
+	if ( fseek( File, 0, SEEK_SET) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
 	}
     }
 
@@ -488,6 +523,19 @@ TypedStreamInput
   return value;
 }
 
+std::string
+TypedStreamInput
+::ReadStdString( const char* key, const std::string& defaultValue, const bool forward )
+{
+  char *value;
+  if ( this->GenericReadArray( key, Self::TYPE_STRING, &value, 1, forward ) != Self::CONDITION_OK )
+    {
+    return std::string( defaultValue );
+    }
+  
+  return std::string( value );
+}
+
 TypedStreamInput::Condition
 TypedStreamInput
 ::GenericReadArray( const char * key, const int type, void *const array, const int arraySize, const bool forward )
@@ -504,15 +552,41 @@ TypedStreamInput
     if ( GzFile ) 
       {
       if ( currentLevel )
-	gzseek( GzFile, LevelStack.top(), SEEK_SET );
+	{
+	if ( -1 == gzseek( GzFile, LevelStack.top(), SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
+	}
       else
-	gzseek( GzFile, 0, SEEK_SET );
+	{
+	if ( -1 == gzseek( GzFile, 0, SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
+	}	  
       }
     else
+      {
       if ( currentLevel )
-	fseek( File, LevelStack.top(), SEEK_SET );
+	{
+	if ( fseek( File, LevelStack.top(), SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
+	}
       else
-	fseek( File, 0, SEEK_SET );
+	{
+	if ( fseek( File, 0, SEEK_SET ) )
+	  {
+	  this->m_Status = Self::ERROR_SYSTEM;
+	  return Self::CONDITION_ERROR;
+	  }
+	}
+      }
     }
   
   int line;
@@ -593,19 +667,22 @@ TypedStreamInput
 	  do 
 	    {
 	    buffer = strtok( BufferValue, "\t\n " );
-	    int idx = 0;
-	    while ( (buffer[idx]=='0') || (buffer[idx]=='1') ) 
+	    if ( buffer )
 	      {
-	      if ( i >= arraySize )
-		return Self::CONDITION_OK;
-	      if ( buffer[idx] == '0' )
-		arrayInt[i/8] &= ~(1<<(i%8));
-	      else
-		arrayInt[i/8] |= (1<<(i%8));
-	      ++i;
-	      ++idx;
+	      int idx = 0;
+	      while ( (buffer[idx]=='0') || (buffer[idx]=='1') ) 
+		{
+		if ( i >= arraySize )
+		  return Self::CONDITION_OK;
+		if ( buffer[idx] == '0' )
+		  arrayInt[i/8] &= ~(1<<(i%8));
+		else
+		  arrayInt[i/8] |= (1<<(i%8));
+		++i;
+		++idx;
+		}
 	      }
-	    } while ( i < arraySize && Self::TOKEN_VALUE == this->ReadLineToken() );
+	    } while ( buffer && i < arraySize && Self::TOKEN_VALUE == this->ReadLineToken() );
 	  if ( i < arraySize ) 
 	    {
 	    this->m_Status = Self::ERROR_ARG;
@@ -681,7 +758,7 @@ TypedStreamInput
 		return Self::CONDITION_ERROR;
 		}
 	      char *b;
-	      if (!(b = (char *)malloc(strlen(buffer)))) 
+	      if (!(b = (char *)malloc(1+strlen(buffer)))) 
 		{
 		for (--i; i >= 0; i--)
 		  free( arrayString[i] );
