@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4447 $
+//  $Revision: 5078 $
 //
-//  $LastChangedDate: 2012-07-06 13:51:04 -0700 (Fri, 06 Jul 2012) $
+//  $LastChangedDate: 2013-12-02 16:26:25 -0800 (Mon, 02 Dec 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -43,6 +43,7 @@
 #include <Base/cmtkAnatomicalOrientation.h>
 
 #include "nifti1.h"
+#include "nifti1_io_math.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -63,6 +64,43 @@ cmtk
 /** \addtogroup IO */
 //@{
 
+/// Helper function - make qform in NIFTI header from affine matrix
+void
+__matrixToNiftiQform( nifti_1_header& header, const AffineXform::MatrixType matrix )
+{  
+  // transpose and put into nifti_io's mat44 struct
+  mat44 R;
+  for ( int j = 0; j < 4; ++j )
+    {
+    for ( int i = 0; i < 4; ++i )
+      {
+      R.m[i][j] = matrix[j][i];
+      }
+    }
+
+  // compute quaternion representation
+  float qb, qc, qd, qx, qy, qz, dx, dy, dz, qfac;
+  nifti_mat44_to_quatern( R, &qb, &qc, &qd, &qx, &qy, &qz, &dx, &dy, &dz, &qfac ) ;
+  
+  // set header fields
+  header.pixdim[0] = qfac;
+  header.quatern_b = qb;
+  header.quatern_c = qc;
+  header.quatern_d = qd;
+  header.qoffset_x = qx;
+  header.qoffset_y = qy;
+  header.qoffset_z = qz;
+}
+
+/**\note If the imported NIFTI file contains a qform and/or sform, and the resulting image object is later written back to a new NIFTI file (via VolumeFromFile::WriteNifti), then these transformations will be stored in the output file's qform and sform fields, 
+  * respectively.
+  * \note If the header contains only a qform, then the qform will be used to initialize cmtk::UniformVolume::m_IndexToPhysicalMatrix, which determines the anatomy-based gross alignment of the image, regardless of qform_code
+  * \note If the header contains only an sform, then the sform will be used instead, regardless of sform_code.
+  * \note If the header contains both sform and qform, then the transformation is used whose code is NIFTI_XFORM_SCANNER_ANAT. If both sform_code and qform_code are NIFTI_XFORM_SCANNER_ANAT, then qform is used.
+  * \note If the header contains neither a qform or an sform, then it is assumed that the image is oriented "RAS", i.e., fastest-moving array index from Left to Right, second fastest from Posterior to Anterior, and third fastest from Inferior to Superior.
+  *\warning In keeping with the nifti1.h documentation, it is assumed that qform_code and sform_code will always be non-negative. If this assumption is violated, qform and sform will be switched in the output file (as written by VolumeFromFile::WriteNifti), or
+  *   one of them may be missing.
+  */
 const UniformVolume::SmartPtr
 VolumeFromFile::ReadNifti( const std::string& pathHdr, const bool detached, const bool readData )
 {
@@ -116,70 +154,65 @@ VolumeFromFile::ReadNifti( const std::string& pathHdr, const bool detached, cons
   volume->SetMetaInfo( META_SPACE_ORIGINAL, niftiSpace );
 
   const short qform_code = header.GetField<short>( offsetof(nifti_1_header,qform_code) );
-  if ( qform_code > 0 )
+  const short sform_code = header.GetField<short>( offsetof(nifti_1_header,sform_code) );
+
+  if ( sform_code > NIFTI_XFORM_UNKNOWN )
     {
-    const float qb = header.GetField<float>( offsetof(nifti_1_header,quatern_b) );
-    const float qc = header.GetField<float>( offsetof(nifti_1_header,quatern_c) );
-    const float qd = header.GetField<float>( offsetof(nifti_1_header,quatern_d) );
-    const double qa = sqrt( std::max( 0.0, 1.0 - (qb*qb + qc*qc + qd*qd) ) );
-
-    const float qfac = (header.GetField<float>( offsetof(nifti_1_header,pixdim) ) >= 0) ? 1.0f : -1.0f;
-
-    const Types::Coordinate directions[3][3] = 
-      {
-	{        pixelDim[0] * (qa*qa+qb*qb-qc*qc-qd*qd),        pixelDim[0] * (2*qb*qc+2*qa*qd),                pixelDim[0] * (2*qb*qd-2*qa*qc) },
-	{        pixelDim[1] * (2*qb*qc-2*qa*qd),                pixelDim[1] * (qa*qa+qc*qc-qb*qb-qd*qd),        pixelDim[1] * (2*qc*qd+2*qa*qb) },
-	{ qfac * pixelDim[2] * (2*qb*qd+2*qa*qc),         qfac * pixelDim[2] * (2*qc*qd-2*qa*qb),         qfac * pixelDim[2] * (qa*qa+qd*qd-qc*qc-qb*qb) }
-      };
+    float srow_x[4], srow_y[4], srow_z[4];
+    header.GetArray( srow_x, offsetof(nifti_1_header,srow_x), 4 );
+    header.GetArray( srow_y, offsetof(nifti_1_header,srow_y), 4 );
+    header.GetArray( srow_z, offsetof(nifti_1_header,srow_z), 4 );
     
-    const Matrix3x3<Types::Coordinate> m3( directions );
-    Matrix4x4<Types::Coordinate> m4( m3 );
-
-    m4[3][0] = header.GetField<float>( offsetof(nifti_1_header,qoffset_x) );
-    m4[3][1] = header.GetField<float>( offsetof(nifti_1_header,qoffset_y) );
-    m4[3][2] = header.GetField<float>( offsetof(nifti_1_header,qoffset_z) );
-
-    volume->m_IndexToPhysicalMatrix = m4;
-    
-    char orientationImage[4];
-    AnatomicalOrientation::GetOrientationFromDirections( orientationImage, m4, niftiSpace );
-    volume->SetMetaInfo( META_IMAGE_ORIENTATION, orientationImage );
-    volume->SetMetaInfo( META_IMAGE_ORIENTATION_ORIGINAL, orientationImage );
-    }
-  else
-    {
-    const short sform_code = header.GetField<short>( offsetof(nifti_1_header,sform_code) );
-    if ( sform_code > 0 )
-      {
-      float srow_x[4], srow_y[4], srow_z[4];
-      header.GetArray( srow_x, offsetof(nifti_1_header,srow_x), 4 );
-      header.GetArray( srow_y, offsetof(nifti_1_header,srow_y), 4 );
-      header.GetArray( srow_z, offsetof(nifti_1_header,srow_z), 4 );
-
-      const Types::Coordinate directions[4][4] = 	
+    const Types::Coordinate directions[4][4] = 	
       {
 	{ srow_x[0], srow_y[0], srow_z[0], 0 },
 	{ srow_x[1], srow_y[1], srow_z[1], 0 },
 	{ srow_x[2], srow_y[2], srow_z[2], 0 },
 	{ srow_x[3], srow_y[3], srow_z[3], 1 }
       };
-      
-      Matrix4x4<Types::Coordinate> m4( directions );      
-      volume->m_IndexToPhysicalMatrix = m4;
     
-      char orientationImage[4];
-      AnatomicalOrientation::GetOrientationFromDirections( orientationImage, m4, niftiSpace );
-      volume->SetMetaInfo( META_IMAGE_ORIENTATION, orientationImage );
-      volume->SetMetaInfo( META_IMAGE_ORIENTATION_ORIGINAL, orientationImage );
-      }
-    else
-      {
-      // no orientation info, default to nifti standard space
-      volume->SetMetaInfo( META_IMAGE_ORIENTATION, niftiSpace );
-      volume->SetMetaInfo( META_IMAGE_ORIENTATION_ORIGINAL, niftiSpace );
-      }
+    Matrix4x4<Types::Coordinate> m4( directions );      
+    if ( (sform_code == NIFTI_XFORM_SCANNER_ANAT) || (qform_code == NIFTI_XFORM_UNKNOWN) )
+      volume->m_IndexToPhysicalMatrix = m4;
+
+    // NIFTI says, sform_code must be > 0, so should be safe to use "abs" and make negative to distinguish from qform (below)
+    volume->m_AlternativeIndexToPhysicalMatrices[-abs(sform_code)] = m4;
     }
   
+  if ( qform_code > NIFTI_XFORM_UNKNOWN )
+    {
+    const float qb = header.GetField<float>( offsetof(nifti_1_header,quatern_b) );
+    const float qc = header.GetField<float>( offsetof(nifti_1_header,quatern_c) );
+    const float qd = header.GetField<float>( offsetof(nifti_1_header,quatern_d) );
+
+    const float qx = header.GetField<float>( offsetof(nifti_1_header,qoffset_x) );
+    const float qy = header.GetField<float>( offsetof(nifti_1_header,qoffset_y) );
+    const float qz = header.GetField<float>( offsetof(nifti_1_header,qoffset_z) );
+
+    const double qfac = (header.GetField<float>( offsetof(nifti_1_header,pixdim) ) >= 0) ? 1.0f : -1.0f;
+    const mat44 m44 = nifti_quatern_to_mat44( qb, qc, qd, qx, qy, qz, pixelDim[0], pixelDim[1], pixelDim[2], qfac );
+
+    Matrix4x4<Types::Coordinate> m4;
+    for ( int j = 0; j < 4; ++j )
+      {
+      for ( int i = 0; i < 4; ++i )
+	{
+	m4[i][j] = m44.m[j][i];
+	}
+      }
+
+    // qform overrides sform if both exist
+    if ( (qform_code == NIFTI_XFORM_SCANNER_ANAT) || (sform_code != NIFTI_XFORM_SCANNER_ANAT) )
+      volume->m_IndexToPhysicalMatrix = m4;
+    // NIFTI says, qform_code must be > 0, so should be safe to use "abs"
+    volume->m_AlternativeIndexToPhysicalMatrices[abs(qform_code)] = m4;
+    }
+
+  char orientationImage[4];
+  AnatomicalOrientation::GetOrientationFromDirections( orientationImage, volume->m_IndexToPhysicalMatrix, niftiSpace );
+  volume->SetMetaInfo( META_IMAGE_ORIENTATION, orientationImage );
+  volume->SetMetaInfo( META_IMAGE_ORIENTATION_ORIGINAL, orientationImage );
+  
   if ( header.GetField<char>( 148 ) )
     {
     char desc[81];
@@ -321,15 +354,54 @@ VolumeFromFile::WriteNifti
   header.dim[6] = 0;
   header.dim[7] = 0;
 
-  header.qform_code = 0;
-  header.sform_code = 1;
-
-  AffineXform::MatrixType m4 = volume.m_IndexToPhysicalMatrix;
-  for ( int i = 0; i < 4; ++i )
+  header.pixdim[0] = 1.0;
+  header.pixdim[1] = static_cast<float>( writeVolume->m_Delta[AXIS_X] );
+  header.pixdim[2] = static_cast<float>( writeVolume->m_Delta[AXIS_Y] );
+  header.pixdim[3] = static_cast<float>( writeVolume->m_Delta[AXIS_Z] );
+  header.pixdim[4] = 0.0;
+  header.pixdim[5] = 0.0;
+  
+  for ( std::map<int,cmtk::AffineXform::MatrixType>::const_iterator it = volume.m_AlternativeIndexToPhysicalMatrices.begin(); it != volume.m_AlternativeIndexToPhysicalMatrices.end(); ++it )
     {
-    header.srow_x[i] = static_cast<float>( m4[i][0] );
-    header.srow_y[i] = static_cast<float>( m4[i][1] );
-    header.srow_z[i] = static_cast<float>( m4[i][2] );
+    const AffineXform::MatrixType m4 = it->second;
+
+    // this came from a qform
+    if ( it->first > 0 ) 
+      {
+      header.qform_code = it->first;
+      __matrixToNiftiQform( header, it->second );
+      }
+    
+    // this came from an sform
+    if ( it->first < 0 ) 
+      {
+      header.sform_code = abs( it->first );
+      for ( int i = 0; i < 4; ++i )
+	{
+	header.srow_x[i] = static_cast<float>( m4[i][0] );
+	header.srow_y[i] = static_cast<float>( m4[i][1] );
+	header.srow_z[i] = static_cast<float>( m4[i][2] );
+	}
+      }
+    }
+
+  // fallback - we want at least a generic qform to be set to the volume's index-to-physical matrix
+  if ( ! (header.qform_code || header.sform_code) )
+    {
+#ifdef IGNORE
+    // This piece of code, when replacing the next two lines (qform stuff) would make CMTK entirely backward-compatible (release 2.3 and earlier), but non-NIFTI-compliant.
+    const AffineXform::MatrixType m4 = volume.m_IndexToPhysicalMatrix;
+      header.sform_code = NIFTI_XFORM_SCANNER_ANAT;
+      for ( int i = 0; i < 4; ++i )
+	{
+	header.srow_x[i] = static_cast<float>( m4[i][0] );
+	header.srow_y[i] = static_cast<float>( m4[i][1] );
+	header.srow_z[i] = static_cast<float>( m4[i][2] );
+	}
+#else
+      header.qform_code = NIFTI_XFORM_SCANNER_ANAT;
+      __matrixToNiftiQform( header, volume.m_IndexToPhysicalMatrix );
+#endif
     }
   
   switch ( data->GetType() ) 
@@ -372,13 +444,6 @@ VolumeFromFile::WriteNifti
       break;
     }  
   
-  header.pixdim[0] = 1;
-  header.pixdim[1] = static_cast<float>( writeVolume->m_Delta[AXIS_X] );
-  header.pixdim[2] = static_cast<float>( writeVolume->m_Delta[AXIS_Y] );
-  header.pixdim[3] = static_cast<float>( writeVolume->m_Delta[AXIS_Z] );
-  header.pixdim[4] = 0.0;
-  header.pixdim[5] = 0.0;
-  
   // determine data range;
   const Types::DataItemRange dataRange = data->GetRange();
   header.cal_max = static_cast<float>( dataRange.m_UpperBound );
