@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4344 $
+//  $Revision: 4939 $
 //
-//  $LastChangedDate: 2012-05-11 14:51:08 -0700 (Fri, 11 May 2012) $
+//  $LastChangedDate: 2013-10-04 10:49:00 -0700 (Fri, 04 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -97,7 +97,16 @@ XformIO::Read( const std::string& path )
     stream.Open( realPath );
 
     AffineXform affineXform;
-    stream >> affineXform;
+    try 
+      {
+      stream >> affineXform;
+      }
+    catch ( const cmtk::Exception& ex )
+      {
+      StdErr << "ERROR: " << ex.what() << "\n";
+      return Xform::SmartPtr( NULL );
+      }
+    
     return Xform::SmartPtr( new AffineXform( affineXform ) );
     }
     default:
@@ -164,6 +173,7 @@ XformIO::Write
     }
     break;
     default:
+      // cannot really get here, but gcc doesn't know that
       break;
     }
 }
