@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2011, 2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3193 $
+//  $Revision: 4918 $
 //
-//  $LastChangedDate: 2011-04-28 16:15:14 -0700 (Thu, 28 Apr 2011) $
+//  $LastChangedDate: 2013-10-03 13:01:32 -0700 (Thu, 03 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -106,188 +106,6 @@ void internalhsevdladiv(const ap::real_value_type& a,
      ap::real_value_type& p,
      ap::real_value_type& q);
 
-/*************************************************************************
-Finding eigenvalues and eigenvectors of a general matrix
-
-The algorithm finds eigenvalues and eigenvectors of a general matrix by
-using the QR algorithm with multiple shifts. The algorithm can find
-eigenvalues and both left and right eigenvectors.
-
-The right eigenvector is a vector x such that A*x = w*x, and the left
-eigenvector is a vector y such that y'*A = w*y' (here y' implies a complex
-conjugate transposition of vector y).
-
-Input parameters:
-    A       -   matrix. Array whose indexes range within [0..N-1, 0..N-1].
-    N       -   size of matrix A.
-    VNeeded -   flag controlling whether eigenvectors are needed or not.
-                If VNeeded is equal to:
-                 * 0, eigenvectors are not returned;
-                 * 1, right eigenvectors are returned;
-                 * 2, left eigenvectors are returned;
-                 * 3, both left and right eigenvectors are returned.
-
-Output parameters:
-    WR      -   real parts of eigenvalues.
-                Array whose index ranges within [0..N-1].
-    WR      -   imaginary parts of eigenvalues.
-                Array whose index ranges within [0..N-1].
-    VL, VR  -   arrays of left and right eigenvectors (if they are needed).
-                If WI[i]=0, the respective eigenvalue is a real number,
-                and it corresponds to the column number I of matrices VL/VR.
-                If WI[i]>0, we have a pair of complex conjugate numbers with
-                positive and negative imaginary parts:
-                    the first eigenvalue WR[i] + sqrt(-1)*WI[i];
-                    the second eigenvalue WR[i+1] + sqrt(-1)*WI[i+1];
-                    WI[i]>0
-                    WI[i+1] = -WI[i] < 0
-                In that case, the eigenvector  corresponding to the first
-                eigenvalue is located in i and i+1 columns of matrices
-                VL/VR (the column number i contains the real part, and the
-                column number i+1 contains the imaginary part), and the vector
-                corresponding to the second eigenvalue is a complex conjugate to
-                the first vector.
-                Arrays whose indexes range within [0..N-1, 0..N-1].
-
-Result:
-    True, if the algorithm has converged.
-    False, if the algorithm has not converged.
-
-Note 1:
-    Some users may ask the following question: what if WI[N-1]>0?
-    WI[N] must contain an eigenvalue which is complex conjugate to the
-    N-th eigenvalue, but the array has only size N?
-    The answer is as follows: such a situation cannot occur because the
-    algorithm finds a pairs of eigenvalues, therefore, if WI[i]>0, I is
-    strictly less than N-1.
-
-Note 2:
-    The algorithm performance depends on the value of the internal parameter
-    NS of the InternalSchurDecomposition subroutine which defines the number
-    of shifts in the QR algorithm (similarly to the block width in block-matrix
-    algorithms of linear algebra). If you require maximum performance
-    on your machine, it is recommended to adjust this parameter manually.
-
-
-See also the InternalTREVC subroutine.
-
-The algorithm is based on the LAPACK 3.0 library.
-*************************************************************************/
-bool rmatrixevd(ap::real_2d_array a,
-     int n,
-     int vneeded,
-     ap::real_1d_array& wr,
-     ap::real_1d_array& wi,
-     ap::real_2d_array& vl,
-     ap::real_2d_array& vr)
-{
-    bool result;
-    ap::real_2d_array a1;
-    ap::real_2d_array vl1;
-    ap::real_2d_array vr1;
-    ap::real_1d_array wr1;
-    ap::real_1d_array wi1;
-    int i;
-//    ap::real_value_type mx;
-
-    ap::ap_error::make_assertion(vneeded>=0&&vneeded<=3, "RMatrixEVD: incorrect VNeeded!");
-    a1.setbounds(1, n, 1, n);
-    for(i = 1; i <= n; i++)
-    {
-        ap::vmove(&a1(i, 1), &a(i-1, 0), ap::vlen(1,n));
-    }
-    result = nonsymmetricevd(a1, n, vneeded, wr1, wi1, vl1, vr1);
-    if( result )
-    {
-        wr.setbounds(0, n-1);
-        wi.setbounds(0, n-1);
-        ap::vmove(&wr(0), &wr1(1), ap::vlen(0,n-1));
-        ap::vmove(&wi(0), &wi1(1), ap::vlen(0,n-1));
-        if( vneeded==2||vneeded==3 )
-        {
-            vl.setbounds(0, n-1, 0, n-1);
-            for(i = 0; i <= n-1; i++)
-            {
-                ap::vmove(&vl(i, 0), &vl1(i+1, 1), ap::vlen(0,n-1));
-            }
-        }
-        if( vneeded==1||vneeded==3 )
-        {
-            vr.setbounds(0, n-1, 0, n-1);
-            for(i = 0; i <= n-1; i++)
-            {
-                ap::vmove(&vr(i, 0), &vr1(i+1, 1), ap::vlen(0,n-1));
-            }
-        }
-    }
-    return result;
-}
-
-
-/*************************************************************************
-Obsolete 1-based subroutine
-*************************************************************************/
-bool nonsymmetricevd(ap::real_2d_array a,
-     int n,
-     int vneeded,
-     ap::real_1d_array& wr,
-     ap::real_1d_array& wi,
-     ap::real_2d_array& vl,
-     ap::real_2d_array& vr)
-{
-    bool result;
-    ap::real_2d_array s;
-    ap::real_1d_array tau;
-    ap::boolean_1d_array sel;
-    int i;
-    int info;
-    int m;
-
-    ap::ap_error::make_assertion(vneeded>=0&&vneeded<=3, "NonSymmetricEVD: incorrect VNeeded!");
-    if( vneeded==0 )
-    {
-        
-        //
-        // Eigen values only
-        //
-        toupperhessenberg(a, n, tau);
-        internalschurdecomposition(a, n, 0, 0, wr, wi, s, info);
-        result = info==0;
-        return result;
-    }
-    
-    //
-    // Eigen values and vectors
-    //
-    toupperhessenberg(a, n, tau);
-    unpackqfromupperhessenberg(a, n, tau, s);
-    internalschurdecomposition(a, n, 1, 1, wr, wi, s, info);
-    result = info==0;
-    if( !result )
-    {
-        return result;
-    }
-    if( vneeded==1||vneeded==3 )
-    {
-        vr.setbounds(1, n, 1, n);
-        for(i = 1; i <= n; i++)
-        {
-            ap::vmove(&vr(i, 1), &s(i, 1), ap::vlen(1,n));
-        }
-    }
-    if( vneeded==2||vneeded==3 )
-    {
-        vl.setbounds(1, n, 1, n);
-        for(i = 1; i <= n; i++)
-        {
-            ap::vmove(&vl(i, 1), &s(i, 1), ap::vlen(1,n));
-        }
-    }
-    internaltrevc(a, n, vneeded, 1, sel, vl, vr, m, info);
-    result = info==0;
-    return result;
-}
-
 
 void internaltrevc(const ap::real_2d_array& t,
      int n,
