@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4398 $
+//  $Revision: 5129 $
 //
-//  $LastChangedDate: 2012-05-31 11:48:23 -0700 (Thu, 31 May 2012) $
+//  $LastChangedDate: 2014-01-09 13:48:15 -0800 (Thu, 09 Jan 2014) $
 //
-//  $LastChangedBy: torsten_at_home $
+//  $LastChangedBy: torstenrohlfing $
 //
 */
 
@@ -90,13 +90,6 @@ cmtk
 AffineRegistrationCommandLine
 ::AffineRegistrationCommandLine 
 ( const int argc, const char* argv[] ) 
-  : m_InitialXformPath( NULL ),
-    m_ReformattedImagePath( NULL ),
-    m_OutputPathITK( NULL ),
-#ifdef CMTK_USE_SQLITE
-    m_UpdateDB( NULL ),
-#endif
-    Protocol( NULL )
 {
   this->m_Metric = 0;
 
@@ -105,15 +98,12 @@ AffineRegistrationCommandLine
   this->m_Exploration = 8;
   this->m_Accuracy = 0.1;
   this->m_Sampling = 1.0;
-  OutParametersName = OutMatrixName = Studylist = Time = NULL;
   InitXlate = 0;
   this->m_NoSwitch = 0;
 
-  const char *InitialStudylist = NULL;
-  Study1 = Study2 = NULL;
-
-  const char* clArg1 = NULL; // input studylist or reference image
-  const char* clArg2 = NULL; // empty or floating image
+  std::string InitialStudylist;
+  std::string clArg1; // input studylist or reference image
+  std::string clArg2; // empty or floating image
 
   try 
     {
@@ -205,16 +195,16 @@ AffineRegistrationCommandLine
 
   this->SetInitialTransformation( AffineXform::SmartPtr( new AffineXform() ) );
     
-  if ( clArg2 ) 
+  if ( ! clArg2.empty() ) 
     {
-    Study1 = const_cast<char*>( clArg1 );
-    Study2 = const_cast<char*>( clArg2 );
+    Study1 = clArg1;
+    Study2 = clArg2;
     } 
   else
     {
     this->m_InitialXformPath = clArg1;
 
-    if ( InitialStudylist ) 
+    if ( ! InitialStudylist.empty() ) 
       {
       StdErr << "WARNING: transformation of input studylist will be overriden by transformation provided with '--initial'.\n";
       }
@@ -229,9 +219,9 @@ AffineRegistrationCommandLine
       }
 
     typedStream.Seek ( "registration" );
-    Study1 = typedStream.ReadString( "reference_study" );
-    Study2 = typedStream.ReadString( "floating_study" );
-    if ( Study2 )
+    Study1 = typedStream.ReadStdString( "reference_study" );
+    Study2 = typedStream.ReadStdString( "floating_study" );
+    if ( ! Study2.empty() )
       {
       AffineXform::SmartPtr affineXform;
       typedStream >> affineXform;
@@ -240,28 +230,36 @@ AffineRegistrationCommandLine
     else
       {
       // legacy studylists have inverse transformation in them
-      Study2 = typedStream.ReadString( "model_study" );
+      Study2 = typedStream.ReadStdString( "model_study" );
       AffineXform::SmartPtr affineXform;
       typedStream >> affineXform;
-      this->SetInitialTransformation( affineXform->GetInverse() );
+      try
+	{
+	this->SetInitialTransformation( affineXform->GetInverse() );
+	}
+      catch ( const AffineXform::MatrixType::SingularMatrixException& )
+	{
+	StdErr << "ERROR: singular matrix read from initialization file\n";
+	throw ExitException( 1 );
+	}
       }
 
     typedStream.Close();
     }
 
-  if ( !Study1 )
+  if ( Study1.empty() )
     {
     StdErr << "ERROR: reference image path resolved to NULL.\n";
     throw cmtk::ExitException( 1 );
     }
   
-  if ( !Study2 )
+  if ( Study2.empty() )
     {
     StdErr << "ERROR: floating image path resolved to NULL.\n";
     throw cmtk::ExitException( 1 );
     }
   
-  UniformVolume::SmartPtr volume( VolumeIO::ReadOriented( Study1 ) );
+  UniformVolume::SmartPtr volume( VolumeIO::ReadOriented( this->Study1 ) );
   if ( !volume )
     {
     StdErr << "ERROR: volume " << this->Study1 << " could not be read\n";
@@ -269,7 +267,7 @@ AffineRegistrationCommandLine
     }
   this->SetVolume_1( UniformVolume::SmartPtr( this->m_PreprocessorRef.GetProcessedImage( volume ) ) );
 
-  volume = UniformVolume::SmartPtr( VolumeIO::ReadOriented( Study2 ) );
+  volume = UniformVolume::SmartPtr( VolumeIO::ReadOriented( this->Study2 ) );
   if ( !volume )
     {
     StdErr << "ERROR: volume " << this->Study2 << " could not be read\n";
@@ -277,7 +275,7 @@ AffineRegistrationCommandLine
     }
   this->SetVolume_2(  UniformVolume::SmartPtr( this->m_PreprocessorFlt.GetProcessedImage( volume ) ) );
 
-  if ( InitialStudylist ) 
+  if ( ! InitialStudylist.empty() ) 
     {
     this->m_InitialXformPath = InitialStudylist;
     Xform::SmartPtr xform( XformIO::Read( InitialStudylist ) );
@@ -296,9 +294,17 @@ AffineRegistrationCommandLine
 
     if ( affine->GetMetaInfo( META_SPACE ) != AnatomicalOrientation::ORIENTATION_STANDARD )
       {
-      TransformChangeFromSpaceAffine toStandardSpace( *affine, *(this->m_Volume_1), *(this->m_Volume_2) );
-      *affine = toStandardSpace.GetTransformation();
-      affine->SetMetaInfo( META_SPACE, AnatomicalOrientation::ORIENTATION_STANDARD );
+      try
+	{
+	TransformChangeFromSpaceAffine toStandardSpace( *affine, *(this->m_Volume_1), *(this->m_Volume_2) );
+	*affine = toStandardSpace.GetTransformation();
+	affine->SetMetaInfo( META_SPACE, AnatomicalOrientation::ORIENTATION_STANDARD );
+	}
+      catch ( const AffineXform::MatrixType::SingularMatrixException& )
+	{
+	StdErr << "ERROR: singular matrix cannot be inverted to change transformation to standard space in AffineRegistrationCommandLine constructor.\n";
+	throw ExitException( 1 );
+	}
       }
     
     this->SetInitialTransformation( affine );
@@ -306,7 +312,7 @@ AffineRegistrationCommandLine
   
   if ( InitXlate ) 
     {
-    if ( this->m_InitialXformPath ) 
+    if ( ! this->m_InitialXformPath.empty() ) 
       {
       StdErr << "WARNING: Initial transformation was taken from studylist. Switch --initxlate / -i will be ignored.\n";
       } 
@@ -326,7 +332,7 @@ AffineRegistrationCommandLine
     this->m_Exploration = maxDelta * (1<<(this->m_AutoMultiLevels-1));
     }
   
-  if ( Protocol ) 
+  if ( !Protocol.empty() ) 
     {
     RegistrationCallback::SmartPtr callback( new ProtocolCallback( Protocol ) );
     this->SetCallback( callback );
@@ -341,11 +347,11 @@ AffineRegistrationCommandLine::InitRegistration ()
 }
 	
 void
-AffineRegistrationCommandLine::OutputResultMatrix( const char* matrixName ) const
+AffineRegistrationCommandLine::OutputResultMatrix( const std::string& matrixName ) const
 {
   const AffineXform::MatrixType& matrix = this->GetTransformation()->Matrix;
   
-  FILE* mfile = fopen( matrixName, "w" );
+  FILE* mfile = fopen( matrixName.c_str(), "w" );
   if ( mfile )
     {
     for ( int i = 0; i < 4; ++i )
@@ -358,19 +364,19 @@ AffineRegistrationCommandLine::OutputResultMatrix( const char* matrixName ) cons
 
 void
 AffineRegistrationCommandLine::OutputResultParameters
-( const char* paramsName, const CoordinateVector& v ) const
+( const std::string& paramsName, const CoordinateVector& v ) const
 {
-  FILE* pfile = fopen( paramsName, "w" );
+  FILE* pfile = fopen( paramsName.c_str(), "w" );
   if ( pfile )
     {
     for ( unsigned int idx=0; idx < v.Dim; ++idx )
-      fprintf( pfile, "#%d: %f\n", idx, v.Elements[idx] );
+      fprintf( pfile, "#%u: %f\n", idx, v.Elements[idx] );
     fclose( pfile );
     }
 }
 
 void
-AffineRegistrationCommandLine::OutputResultList( const char* studyList ) const
+AffineRegistrationCommandLine::OutputResultList( const std::string& studyList ) const
 {
   ClassStreamOutput classStream( studyList, "studylist", ClassStreamOutput::MODE_WRITE );
   if ( !classStream.IsValid() ) return;
@@ -437,69 +443,72 @@ AffineRegistrationCommandLine::OutputResult ( const CoordinateVector* v, const C
 {
   DebugOutput( 1 ) << "Resulting transformation parameters: \n";
   for ( unsigned int idx=0; idx<v->Dim; ++idx )
-    DebugOutput( 1 ).GetStream().printf( "#%d: %f\n", idx, v->Elements[idx] );
+    DebugOutput( 1 ).GetStream().printf( "#%ud: %f\n", idx, v->Elements[idx] );
   
-  if ( this->OutMatrixName )
+  if ( !this->OutMatrixName.empty() )
     {
-    std::string path( this->OutMatrixName );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-
-    this->OutputResultMatrix( path.c_str() );
+      this->OutputResultMatrix( this->OutMatrixName + "-partial" );
+    else
+      this->OutputResultMatrix( this->OutMatrixName );
     }
 
-  if ( this->OutParametersName )
+  if ( !this->OutParametersName.empty() )
     {
-    std::string path( this->OutParametersName );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-    
-    this->OutputResultParameters( path.c_str(), *v );
+      this->OutputResultParameters( this->OutParametersName + "-partial", *v );
+    else
+      this->OutputResultParameters( this->OutParametersName, *v );
     }
 
-  if ( this->Studylist ) 
+  if ( !this->Studylist.empty() ) 
     {
-    std::string path( this->Studylist );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-
-    this->OutputResultList( path.c_str() );
+      this->OutputResultList( this->Studylist + "-partial" );
+    else
+      this->OutputResultList( this->Studylist );
     }
 
-  if ( this->m_OutputPathITK ) 
+  if ( !this->m_OutputPathITK.empty() ) 
     {
-    std::string path( this->m_OutputPathITK );
-    if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-    
-    TransformChangeToSpaceAffine toNative( *(this->GetTransformation()), *(this->m_Volume_1), *(this->m_Volume_2), AnatomicalOrientationBase::SPACE_ITK );
-    AffineXformITKIO::Write( path.c_str(), toNative.GetTransformation() );
+    try
+      {
+      TransformChangeToSpaceAffine toNative( *(this->GetTransformation()), *(this->m_Volume_1), *(this->m_Volume_2), AnatomicalOrientationBase::SPACE_ITK );
+      if ( irq != CALLBACK_OK )
+	AffineXformITKIO::Write( this->m_OutputPathITK + "-partial", toNative.GetTransformation() );
+      else
+	AffineXformITKIO::Write( this->m_OutputPathITK, toNative.GetTransformation() );
+      }
+    catch ( const AffineXform::MatrixType::SingularMatrixException& )
+      {
+      StdErr << "ERROR: singular matrix cannot be inverted to change transformation to standard space in AffineRegistrationCommandLine::OutputResult\n";
+      throw ExitException( 1 );
+      }
     }
   
-  if ( this->m_ReformattedImagePath )
+  if ( !this->m_ReformattedImagePath.empty() )
     {
-    std::string path( this->m_ReformattedImagePath );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-    
-    VolumeIO::Write( *(this->GetReformattedFloatingImage()), path.c_str() );
+      VolumeIO::Write( *(this->GetReformattedFloatingImage()), this->m_ReformattedImagePath + "-partial" );
+    else
+      VolumeIO::Write( *(this->GetReformattedFloatingImage()), this->m_ReformattedImagePath );
     }
 
 #ifdef CMTK_USE_SQLITE
-  if ( this->m_UpdateDB && (irq == CALLBACK_OK) )
+  if ( (irq == CALLBACK_OK) && !this->m_UpdateDB.empty() )
     {
     try
       {
       ImageXformDB db( this->m_UpdateDB );
       
-      if ( this->m_ReformattedImagePath )
+      if ( !this->m_ReformattedImagePath.empty() )
 	{
 	db.AddImage( this->m_ReformattedImagePath, this->m_ReferenceVolume->GetMetaInfo( META_FS_PATH ) );
 	}
       
-      if ( this->Studylist )
+      if ( !this->Studylist.empty() )
 	{
-	if ( this->m_InitialXformPath ) 
+	if ( ! this->m_InitialXformPath.empty() )
 	  {
 	  db.AddRefinedXform( this->Studylist, true /*invertible*/, this->m_InitialXformPath, this->m_InitialXformIsInverse );
 	  }
@@ -533,9 +542,9 @@ AffineRegistrationCommandLine::Register ()
   CallbackResult Result = Superclass::Register();
   const int elapsed = static_cast<int>( Timers::GetTimeProcess() - baselineTime );
 
-  if ( Time ) 
+  if ( !this->Time.empty() ) 
     {
-    FILE *tfp = fopen( Time, "w" );
+    FILE *tfp = fopen( this->Time.c_str(), "w" );
     
     if ( tfp ) 
       {
@@ -544,7 +553,7 @@ AffineRegistrationCommandLine::Register ()
       } 
     else 
       {
-      StdErr << "Could not open time file " << Time << "\n";
+      StdErr << "Could not open time file " << this->Time << "\n";
       }
     }
   return Result;
