@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4423 $
+//  $Revision: 5129 $
 //
-//  $LastChangedDate: 2012-06-11 16:13:37 -0700 (Mon, 11 Jun 2012) $
+//  $LastChangedDate: 2014-01-09 13:48:15 -0800 (Thu, 09 Jan 2014) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -39,6 +39,7 @@
 #include <System/cmtkThreads.h>
 #include <System/cmtkCompressedStream.h>
 #include <System/cmtkMountPoints.h>
+#include <System/cmtkExitException.h>
 
 #include <IO/cmtkXformIO.h>
 #include <IO/cmtkClassStreamInput.h>
@@ -87,28 +88,18 @@ ImagePairNonrigidRegistrationCommandLine* ImagePairNonrigidRegistrationCommandLi
 
 ImagePairNonrigidRegistrationCommandLine
 ::ImagePairNonrigidRegistrationCommandLine
-( const int argc, const char *argv[] ) :
-  m_InitialTransformationFile( NULL ),
-  m_InitialTransformationInverse( false ),
-#ifdef CMTK_USE_SQLITE
-  m_UpdateDB( NULL ),
-#endif
-  m_OutputPathITK( NULL ),
-  m_ReformattedImagePath( NULL )
+( const int argc, const char *argv[] )
 {
-  Studylist = Time = NULL;
-
   this->m_OutputIntermediate = 0;
 
-  InputStudylist = NULL;
   IntermediateResultIndex = 0;
 
   bool forceOutsideFlag = false;
   Types::DataItem forceOutsideValue = 0;
 
-  const char* clArg1 = NULL; // input studylist or reference image
-  const char* clArg2 = NULL; // empty or floating image
-  const char* clArg3 = NULL; // empty or initial transformation
+  std::string clArg1; // input studylist or reference image
+  std::string clArg2; // empty or floating image
+  std::string clArg3; // empty or initial transformation
 
   try
     {
@@ -206,10 +197,8 @@ ImagePairNonrigidRegistrationCommandLine
     cl.EndGroup();
 #endif
 
-    cl.AddParameter( &clArg1, "ReferenceImage", "Reference (fixed) image path" )
-      ->SetProperties( CommandLine::PROPS_IMAGE );
-    cl.AddParameter( &clArg2, "FloatingImage", "Floating (moving) image path" )
-      ->SetProperties( CommandLine::PROPS_IMAGE | CommandLine::PROPS_OPTIONAL);
+    cl.AddParameter( &clArg1, "ReferenceImage", "Reference (fixed) image path" )->SetProperties( CommandLine::PROPS_IMAGE );
+    cl.AddParameter( &clArg2, "FloatingImage", "Floating (moving) image path" )->SetProperties( CommandLine::PROPS_IMAGE | CommandLine::PROPS_OPTIONAL);
     cl.AddParameter( &clArg3, "InitialXform", "Initial affine transformation from reference to floating image" )
       ->SetProperties( CommandLine::PROPS_NOXML | CommandLine::PROPS_XFORM | CommandLine::PROPS_OPTIONAL );
 
@@ -227,14 +216,14 @@ ImagePairNonrigidRegistrationCommandLine
     exit( 1 );
     }
 
-  if ( clArg2 ) 
+  if ( ! clArg2.empty() ) 
     {
     this->SetInitialTransformation( AffineXform::SmartPtr( new AffineXform() ) );
     
-    Study1 = const_cast<char*>( clArg1 );
-    Study2 = const_cast<char*>( clArg2 );
+    this->Study1 = clArg1;
+    this->Study2 = clArg2;
 
-    if ( clArg3 )
+    if ( ! clArg3.empty() )
       {
       this->m_InitialTransformationFile = clArg3;
       }
@@ -253,9 +242,9 @@ ImagePairNonrigidRegistrationCommandLine
       }
     
     classStream.Seek ( "registration" );
-    Study1 = classStream.ReadString( "reference_study" );
-    Study2 = classStream.ReadString( "floating_study" );
-    if ( Study2 )
+    Study1 = classStream.ReadStdString( "reference_study" );
+    Study2 = classStream.ReadStdString( "floating_study" );
+    if ( ! Study2.empty() )
       {
       AffineXform::SmartPtr affineXform;
       classStream >> affineXform;
@@ -264,38 +253,67 @@ ImagePairNonrigidRegistrationCommandLine
     else
       {
       // legacy studylists have inverse transformation stored in them
-      Study2 = classStream.ReadString( "model_study" );
+      Study2 = classStream.ReadStdString( "model_study" );
       AffineXform::SmartPtr affineXform;
       classStream >> affineXform;
-      this->SetInitialTransformation( affineXform->GetInverse() );
+      try
+	{
+	this->SetInitialTransformation( affineXform->GetInverse() );
+	}
+      catch ( const AffineXform::MatrixType::SingularMatrixException& )
+	{
+	StdErr << "ERROR: singular matrix from initialization file in cmtk::ImagePairNonrigidRegistrationCommandLine constructor\n";
+	throw ExitException( 1 );
+	}
       }
     
     classStream.Close();
     }
   
   // Was an initial studylist given? If so, get warp 
-  if ( this->m_InitialTransformationFile ) 
+  if ( ! this->m_InitialTransformationFile.empty() ) 
     {
     Xform::SmartPtr initialXform( XformIO::Read( this->m_InitialTransformationFile ) );
-    AffineXform::SmartPtr affineXform = AffineXform::SmartPtr::DynamicCastFrom( initialXform );
-    if ( affineXform )
+
+    if ( initialXform )
       {
-      if ( this->m_InitialTransformationInverse )
-	this->SetInitialTransformation( affineXform->GetInverse() );
+      AffineXform::SmartPtr affineXform = AffineXform::SmartPtr::DynamicCastFrom( initialXform );
+      if ( affineXform )
+	{
+	if ( this->m_InitialTransformationInverse )
+	  {
+	  try
+	    {
+	    this->SetInitialTransformation( affineXform->GetInverse() );
+	    }
+	  catch ( const AffineXform::MatrixType::SingularMatrixException& )
+	    {
+	    StdErr << "ERROR: singular matrix from initialization file in cmtk::ImagePairNonrigidRegistrationCommandLine constructor\n";
+	    throw ExitException( 1 );
+	    }
+	  }
+	else
+	  {
+	  this->SetInitialTransformation( affineXform );
+	  }
+	}
       else
-	this->SetInitialTransformation( affineXform );
+	{
+	InitialWarpXform = SplineWarpXform::SmartPtr::DynamicCastFrom( initialXform );
+	}
       }
     else
       {
-      InitialWarpXform = SplineWarpXform::SmartPtr::DynamicCastFrom( initialXform );
+      StdErr << "ERROR: could not read initial transformation from " << this->m_InitialTransformationFile << "\n";
+      throw ExitException( 1 );
       }
     }
 
-  UniformVolume::SmartPtr volume( VolumeIO::ReadOriented( Study1 ) );
+  UniformVolume::SmartPtr volume( VolumeIO::ReadOriented( this->Study1 ) );
   if ( !volume ) throw ConstructorFailed();
   this->SetVolume_1( UniformVolume::SmartPtr( this->m_PreprocessorRef.GetProcessedImage( volume ) ) );
 
-  volume = UniformVolume::SmartPtr( VolumeIO::ReadOriented( Study2 ) );
+  volume = UniformVolume::SmartPtr( VolumeIO::ReadOriented( this->Study2 ) );
   if ( !volume ) throw ConstructorFailed();
   this->SetVolume_2( UniformVolume::SmartPtr( this->m_PreprocessorFlt.GetProcessedImage( volume ) ) );
 
@@ -304,10 +322,18 @@ ImagePairNonrigidRegistrationCommandLine
     {
     if ( affineXform->MetaKeyExists( META_SPACE ) && (affineXform->GetMetaInfo( META_SPACE ) != AnatomicalOrientation::ORIENTATION_STANDARD ) )
       {
-      TransformChangeFromSpaceAffine toStandardSpace( *affineXform, *(this->m_Volume_1), *(this->m_Volume_2), affineXform->GetMetaInfo( META_SPACE ).c_str() );
-      *affineXform = toStandardSpace.GetTransformation();
-      affineXform->SetMetaInfo( META_SPACE, AnatomicalOrientation::ORIENTATION_STANDARD );
-      this->SetInitialTransformation( affineXform );
+      try
+	{
+	TransformChangeFromSpaceAffine toStandardSpace( *affineXform, *(this->m_Volume_1), *(this->m_Volume_2), affineXform->GetMetaInfo( META_SPACE ).c_str() );
+	*affineXform = toStandardSpace.GetTransformation();
+	affineXform->SetMetaInfo( META_SPACE, AnatomicalOrientation::ORIENTATION_STANDARD );
+	this->SetInitialTransformation( affineXform );
+	}
+      catch ( const AffineXform::MatrixType::SingularMatrixException& )
+	{
+	StdErr << "ERROR: singular matrix cannot be inverted to change transformation to standard space in ElasticRegistrationCommandLine constructor.\n";
+	throw ExitException( 1 );
+	}
       }
     }
 
@@ -350,54 +376,51 @@ ImagePairNonrigidRegistrationCommandLine
 ::OutputResult
 ( const CoordinateVector*, const CallbackResult irq )
 {
-  if ( this->Studylist ) 
+  if ( !this->Studylist.empty() ) 
     {
-    std::string path( this->Studylist );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-
-    this->OutputWarp( path.c_str() );
+      this->OutputWarp( this->Studylist + "-partial" );
+    else
+      this->OutputWarp( this->Studylist );
     }
 
-  if ( this->m_OutputPathITK ) 
+  if ( !this->m_OutputPathITK.empty() ) 
     {
-    std::string path( this->m_OutputPathITK );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-    
-    SplineWarpXformITKIO::Write( path.c_str(), *(this->GetTransformation()), *(this->m_ReferenceVolume), *(this->m_FloatingVolume) );
+      SplineWarpXformITKIO::Write( this->m_OutputPathITK + "-partial", *(this->GetTransformation()), *(this->m_ReferenceVolume), *(this->m_FloatingVolume) );
+    else
+      SplineWarpXformITKIO::Write( this->m_OutputPathITK, *(this->GetTransformation()), *(this->m_ReferenceVolume), *(this->m_FloatingVolume) );
     }
   
-  if ( this->m_ReformattedImagePath )
+  if ( !this->m_ReformattedImagePath.empty() )
     {
-    std::string path( this->m_ReformattedImagePath );
     if ( irq != CALLBACK_OK )
-      path.append( "-partial" );
-    
-    VolumeIO::Write( *(this->GetReformattedFloatingImage() ), path.c_str() );
+      VolumeIO::Write( *(this->GetReformattedFloatingImage() ), this->m_ReformattedImagePath + "-partial" );
+    else
+      VolumeIO::Write( *(this->GetReformattedFloatingImage() ), this->m_ReformattedImagePath );
     }
 
 #ifdef CMTK_USE_SQLITE
-  if ( this->m_UpdateDB && (irq == CALLBACK_OK) )
+  if ( (irq == CALLBACK_OK) && !this->m_UpdateDB.empty() )
     {
     try
       {
       ImageXformDB db( this->m_UpdateDB );
       
-      if ( this->m_ReformattedImagePath )
+      if ( !this->m_ReformattedImagePath.empty() )
 	{
 	db.AddImage( this->m_ReformattedImagePath, this->m_ReferenceVolume->GetMetaInfo( META_FS_PATH ) );
 	}
       
-      if ( this->Studylist )
+      if ( ! this->Studylist.empty() )
 	{
-	if ( this->InputStudylist ) 
+	if ( ! this->InputStudylist.empty() ) 
 	  {
 	  db.AddRefinedXform( this->Studylist, true /*invertible*/, this->InputStudylist );
 	  }
 	else 
 	  { 
-	  if ( this->m_InitialTransformationFile )
+	  if ( ! this->m_InitialTransformationFile.empty() )
 	    {
 	    db.AddRefinedXform( this->Studylist, true /*invertible*/, this->m_InitialTransformationFile, m_InitialTransformationInverse );
 	    }
@@ -419,8 +442,7 @@ ImagePairNonrigidRegistrationCommandLine
 void
 ImagePairNonrigidRegistrationCommandLine
 ::EnterResolution
-( CoordinateVector::SmartPtr& v, Functional::SmartPtr& f, 
-  const int index, const int total )
+( CoordinateVector::SmartPtr& v, Functional::SmartPtr& f, const int index, const int total )
 {
   DebugOutput( 1 ).GetStream().printf( "\rEntering resolution level %d out of %d...\n", index, total );
   this->Superclass::EnterResolution( v, f, index, total );
@@ -436,7 +458,7 @@ ImagePairNonrigidRegistrationCommandLine::DoneResolution
 }
 
 void
-ImagePairNonrigidRegistrationCommandLine::OutputWarp ( const char* path ) const
+ImagePairNonrigidRegistrationCommandLine::OutputWarp ( const std::string& path ) const
 {
   ClassStreamOutput classStream( path, "studylist", ClassStreamOutput::MODE_WRITE );
   if ( ! classStream.IsValid() ) return;
@@ -531,9 +553,9 @@ void
 ImagePairNonrigidRegistrationCommandLine::OutputIntermediate( const bool incrementCount )
 {
   char path[PATH_MAX];
-  if ( Studylist ) 
+  if ( ! Studylist.empty() ) 
     {
-    snprintf( path, sizeof( path ), "%s%clevel-%02d.list", Studylist, (int)CMTK_PATH_SEPARATOR, IntermediateResultIndex );
+    snprintf( path, sizeof( path ), "%s%clevel-%02d.list", Studylist.c_str(), (int)CMTK_PATH_SEPARATOR, IntermediateResultIndex );
     } 
   else
     {
