@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4375 $
+//  $Revision: 4935 $
 //
-//  $LastChangedDate: 2012-05-30 13:01:18 -0700 (Wed, 30 May 2012) $
+//  $LastChangedDate: 2013-10-04 10:33:33 -0700 (Fri, 04 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -255,8 +255,11 @@ TypedArraySimilarity::GetCrossCorrelation
       }
     }
 
-  mean0 /= count;
-  mean1 /= count;
+  if ( count )
+    {
+    mean0 /= count;
+    mean1 /= count;
+    }
 
   for ( int idx = 0; idx < static_cast<int>( numberOfPixels ); ++idx ) 
     {
