@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4427 $
+//  $Revision: 4847 $
 //
-//  $LastChangedDate: 2012-06-12 11:23:22 -0700 (Tue, 12 Jun 2012) $
+//  $LastChangedDate: 2013-09-13 15:05:13 -0700 (Fri, 13 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -59,7 +59,12 @@ cmtk
 VoxelMatchingElasticFunctional::VoxelMatchingElasticFunctional
 ( UniformVolume::SmartPtr& reference, UniformVolume::SmartPtr& floating )
   : VoxelMatchingFunctional( reference, floating ),
-    m_ActiveCoordinates( NULL )
+    m_ActiveCoordinates( NULL ),
+    m_JacobianConstraintWeight( 0.0 ),
+    m_RigidityConstraintWeight( 0.0 ),
+    m_GridEnergyWeight( 0.0 ),
+    m_Regularize( false ),
+    WarpNeedsFixUpdate( false )
 {
   Dim = 0;
 
