@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3743 $
+//  $Revision: 4381 $
 //
-//  $LastChangedDate: 2012-01-25 15:40:11 -0800 (Wed, 25 Jan 2012) $
+//  $LastChangedDate: 2012-05-30 14:18:15 -0700 (Wed, 30 May 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -49,7 +49,7 @@ VoxelMatchingFunctional::InitFloating( UniformVolume::SmartPtr& floating )
   FloatingGrid = floating;
   
   this->FloatingDims = this->FloatingGrid->GetDims();
-  this->FloatingSize = this->FloatingGrid->Size;
+  this->FloatingSize = this->FloatingGrid->m_Size;
 
   this->m_FloatingCropRegionCoordinates = FloatingGrid->GetHighResCropRegion();
   for ( int dim = 0; dim < 3; ++dim ) 
@@ -68,7 +68,7 @@ VoxelMatchingFunctional::InitReference( UniformVolume::SmartPtr& reference )
   ReferenceGrid = reference;
 
   this->ReferenceDims = this->ReferenceGrid->GetDims();
-  this->ReferenceSize = this->ReferenceGrid->Size;
+  this->ReferenceSize = this->ReferenceGrid->m_Size;
   this->m_ReferenceCropRegion = ReferenceGrid->CropRegion();
 
   for ( int dim = 0; dim < 3; ++dim )
