@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2012 SRI International
+//  Copyright 2012, 2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -20,11 +20,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4326 $
+//  $Revision: 4962 $
 //
-//  $LastChangedDate: 2012-05-08 12:05:35 -0700 (Tue, 08 May 2012) $
+//  $LastChangedDate: 2013-10-10 21:02:01 -0700 (Thu, 10 Oct 2013) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -62,14 +62,98 @@ public:
   /// Smart pointer to const for  this class.
   typedef SmartConstPointer<Self> SmartConstPtr;
 
+  /// Class for parameters that control the phantom detection
+  class Parameters
+  {
+  public:
+    /// Default constructor.
+    Parameters() : 
+      m_CorrectSphereBiasField( true ), 
+      m_TolerateTruncation( false ),
+      m_StandardOrientation( true ),
+      m_BipolarFilterMargin( 1 ), 
+      m_RefineMarginPixels( 1 ), 
+      m_SphereExcludeSafetyMargin( 10.0 ),
+      m_ErodeSNR( 10.0 ), 
+      m_ErodeCNR( 5.0 ),
+      m_RefineXformEachLandmark( false ),
+      m_RefineOutliers( false ),
+      m_ExcludeOutliers( false ),
+      m_LandmarkFitResidualThreshold( 5.0 )
+    {}
+
+    /// Flag for correction of (linear) bias field for each sphere.
+    bool m_CorrectSphereBiasField;
+
+    /** Flag for tolerant operation - if set, we are lenient when spheres are slightly truncated.
+     * This should be considered a last resort, and both phantom scans and results should be carefully inspected.
+     */
+    bool m_TolerateTruncation;
+
+    /** Flag for standard orientation.
+     * Setting this assumes that the phantom was scanned with the correct side up, rather than upside down.
+     * This will make detection of defective phantoms more robust, but it will also prevent detection of
+     * phantoms scanned upside down.
+     */
+    bool m_StandardOrientation;
+
+    /// Margin (in pixels) for the bipolar sphere detection matched filter.
+    int m_BipolarFilterMargin;
+
+    /// Margin in pixels for center-of-mass-based refinement.
+    int m_RefineMarginPixels;
+
+    /// Safety margin (in mm) around detected spheres - no other sphere centers are permitted within this margin.
+    Types::Coordinate m_SphereExcludeSafetyMargin;
+
+    /// Erode SNR sphere by this many pixels for SNR computation
+    Types::Coordinate m_ErodeSNR;
+    
+    /// Erode CNR spheres by this many pixels for SNR computation
+    Types::Coordinate m_ErodeCNR;
+
+    /// Flag for optional refinement of transformation after each new landmark has been added.
+    bool m_RefineXformEachLandmark;
+    
+    /// Flag for optional refinement of outlier landmarks
+    bool m_RefineOutliers;
+
+    /// Flag for optional exclusion of outlier landmarks from the final, fitted transformation
+    bool m_ExcludeOutliers;
+    
+    /// Threshold for detecting outliers based on landmark fitting residuals.
+    Types::Coordinate m_LandmarkFitResidualThreshold;
+
+  private:
+    /// Static default parameters.
+    static Parameters Default;
+
+    /// Let outside class have access.
+    friend class DetectPhantomMagphanEMR051;
+  };
+
   /// Spatial coordinate vector.
   typedef UniformVolume::SpaceVectorType SpaceVectorType;
 
   /// Exception thrown if a landmark sphere cannot be localized in the search region.
   class NoSphereInSearchRegion : public Exception {};
   
+  /// Exception thrown if the field of view is insufficient.
+  class OutsideFieldOfView : public Exception 
+  {
+  public:
+    // Constructor takes index and predicted location of offending landmark.
+    OutsideFieldOfView( const size_t idx, const UniformVolume::CoordinateVectorType& v ) : m_Idx( idx ), m_Location( v ) {}
+
+    // Offending landmark index.
+    size_t m_Idx;
+
+    // Offending predicted location.
+    UniformVolume::CoordinateVectorType m_Location;
+  };
+  
   /// Constructor: detect all landmark spheres.
-  DetectPhantomMagphanEMR051( UniformVolume::SmartConstPtr& phantomImage );
+  DetectPhantomMagphanEMR051( UniformVolume::SmartConstPtr& phantomImage, Self::Parameters& parameters = Self::Parameters::Default );
 
   /// Get comprehensive description of phantom as detected in image.
   DetectedPhantomMagphanEMR051::SmartPtr GetDetectedPhantom();
@@ -96,9 +180,12 @@ public:
   LandmarkList GetDetectedLandmarks( const bool includeOutliers = false /*!< If true, include landmarks detected as outliers based on linear affine transformation fitting residual */ ) const;
 
 private:
-  /// Flag for correction of (linear) bias field for each sphere.
-  bool m_CorrectSphereBiasField;
-  
+  /// Parameters that control the phantom detection
+  Self::Parameters m_Parameters;
+
+  /// Status flags that cover a variety of internal conditions.
+  DetectedPhantomMagphanEMR051::StatusFlags m_StatusFlags;
+
   /// Image of the phantom.
   UniformVolume::SmartConstPtr m_PhantomImage;
 
@@ -112,8 +199,25 @@ private:
    */
   UniformVolume::SmartPtr m_IncludeMask;
 
+  /// Class for landmarks and validity flags
+  class LandmarkType
+  {
+  public:
+    /// Default constructor.
+    LandmarkType() : m_Location( 0 ), m_Valid( false ) {}
+
+    /// Constructor.
+    LandmarkType( const Self::SpaceVectorType& location, const bool valid = true ) : m_Location( location ), m_Valid( valid ) {}
+
+    /// Location of the landmark
+    Self::SpaceVectorType m_Location;
+
+    /// Is this landmark valid?
+    bool m_Valid;
+  };
+
   /// The detected sphere centroid landmarks in image space.
-  std::vector<Self::SpaceVectorType> m_Landmarks;
+  std::vector<Self::LandmarkType> m_Landmarks;
 
   /// Fitted rigid transformation from phantom to image coordinates.
   AffineXform::SmartPtr m_PhantomToImageTransformationRigid;
@@ -142,30 +246,6 @@ private:
   /// Refine sphere position based on intensity-weighted center of mass.
   Self::SpaceVectorType RefineSphereLocation( const Self::SpaceVectorType& estimate, const Types::Coordinate radius, const int label );
 
-  /// Get margin (in pixels) for the bipolar sphere detection matched filter.
-  int GetBipolarFilterMargin() const
-  {
-    return 1;
-  }
-
-  /// Get safety margin (in mm) around detected spheres - no other sphere centers are permitted within this margin.
-  Types::Coordinate GetSphereExcludeSafetyMargin() const
-  {
-    return 10.0;
-  }
-
-  /// Get margin in pixels for center-of-mass-based refinement.
-  int GetRefineMarginPixels() const
-  {
-    return 1;
-  }
-
-  /// Get threshold for detecting outliers based on landmark fitting residuals.
-  Types::Coordinate GetLandmarkFitResidualThreshold() const
-  {
-    return 5.0; // if we're off bymore than 5mm, that's probably not due to actual distortion
-  }
-
   /** Compute landmark fitting residuals under given linear transformation
    *\return The maximum residual over all landmarks. This can be compared with a threshold to
    * determine whether refinement of outliers is necessary.
@@ -181,7 +261,7 @@ private:
   /// Get the mean and standard deviation of intensities within a sphere of given location and radius.
   void GetSphereMeanStdDeviation( Types::DataItem& mean /*!< Reference to return mean intensity */, Types::DataItem& stdev /*!< Reference to return intensity standard deviation */, 
 				  const Self::SpaceVectorType& center /*!< Center coordinate of the sphere. */, const Types::Coordinate radius /*!< Radius of the sphere */, 
-				  const int erodeBy /*!< Number of pixels to erode the sphere by before computing mean and standard deviation. */,
+				  const Types::Coordinate erodeBy /*!< Distance to erode the sphere by before computing mean and standard deviation. */,
 				  const int biasFieldDegree /*!< Polynomial degree of the estimated bias field before computing mean and standard deviation (0 = no bias field correction) */ );
 };
 
