@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2012 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3975 $
+//  $Revision: 4873 $
 //
-//  $LastChangedDate: 2012-03-06 15:04:42 -0800 (Tue, 06 Mar 2012) $
+//  $LastChangedDate: 2013-09-24 12:53:12 -0700 (Tue, 24 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -53,7 +53,7 @@ cmtk
 //@{
 
 LabelCombinationShapeBasedAveragingInterpolation::LabelCombinationShapeBasedAveragingInterpolation
-( const std::vector<UniformVolume::SmartConstPtr>& labelImages, const std::vector<cmtk::XformUniformVolume::SmartConstPtr> xformsToLabelImages, const UniformVolume::SmartConstPtr& targetGrid, 
+( const std::vector<UniformVolume::SmartConstPtr>& labelImages, const std::vector<cmtk::XformUniformVolume::SmartConstPtr>& xformsToLabelImages, const UniformVolume::SmartConstPtr& targetGrid, 
   const Self::LabelIndexType numberOfLabels )
   : LabelCombinationShapeBasedAveraging( labelImages, numberOfLabels ),
     m_TargetGrid( targetGrid ),
