@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2010-2012 SRI International
+//  Copyright 2010-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -20,9 +20,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4111 $
+//  $Revision: 4922 $
 //
-//  $LastChangedDate: 2012-03-30 14:30:21 -0700 (Fri, 30 Mar 2012) $
+//  $LastChangedDate: 2013-10-03 14:51:17 -0700 (Thu, 03 Oct 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -87,7 +87,14 @@ cmtk::SimpleLevelset
 	outsideSum += this->m_Volume->GetDataAt( n );
       }
 
+    if ( nInside == 0 )
+      throw Self::DegenerateLevelsetException();
+
     const size_t nOutside = numberOfPixels - nInside;
+
+    if ( nOutside == 0 )
+      throw Self::DegenerateLevelsetException();
+
     const Types::DataItem ratioInOut = 1.0 * nInside / nOutside;
     
     const Types::DataItem mInside = insideSum / nInside;
