@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2012 SRI International
+//  Copyright 2012, 2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -20,11 +20,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4458 $
+//  $Revision: 4897 $
 //
-//  $LastChangedDate: 2012-07-19 21:04:00 -0700 (Thu, 19 Jul 2012) $
+//  $LastChangedDate: 2013-09-30 14:12:06 -0700 (Mon, 30 Sep 2013) $
 //
-//  $LastChangedBy: torsten_at_home $
+//  $LastChangedBy: torstenrohlfing $
 //
 */
 
@@ -35,18 +35,18 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::SphereDetectionNormalize
       m_ImageDims( image.m_Dims ),
       m_PixelSize( image.m_Delta ),
       m_SphereRadius( 0 ),
-      m_MarginWidth( -1 )
+      m_MarginWidth( -1 ),
+      m_SumFilter( 0.0 ),
+      m_SumFilterMask( 0.0 )
 {
   this->m_ImageFT = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
   this->m_ImageSquareFT = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
 
   this->m_FilterFT = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
-  this->m_FilterSquareFT = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
   this->m_FilterMaskFT = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
   this->m_FilterMaskFT2 = static_cast<fftw_complex*>( fftw_malloc( sizeof( fftw_complex ) * this->m_NumberOfPixels ) );
 
   this->m_PlanFilter = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_FilterFT, this->m_FilterFT, FFTW_FORWARD, FFTW_ESTIMATE );
-  this->m_PlanFilterSquare = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_FilterSquareFT, this->m_FilterSquareFT, FFTW_FORWARD, FFTW_ESTIMATE );
   this->m_PlanFilterMask = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_FilterMaskFT, this->m_FilterMaskFT, FFTW_FORWARD, FFTW_ESTIMATE );
 
   this->m_PlanBackward = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_FilterFT, this->m_FilterFT, FFTW_BACKWARD, FFTW_ESTIMATE );
@@ -56,6 +56,7 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::SphereDetectionNormalize
   // initialize image FT
   fftw_plan plan_image = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_ImageFT, this->m_ImageFT, FFTW_FORWARD, FFTW_ESTIMATE );
   fftw_plan plan_image_square = fftw_plan_dft_3d( this->m_ImageDims[2], this->m_ImageDims[1], this->m_ImageDims[0], this->m_ImageSquareFT, this->m_ImageSquareFT, FFTW_FORWARD, FFTW_ESTIMATE );
+#pragma omp parallel for
   for ( size_t n = 0; n < this->m_NumberOfPixels; ++n )
     {
     this->m_ImageFT[n][0] = image.GetDataAt( n );
@@ -78,13 +79,11 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::~SphereDetectionNormaliz
   fftw_destroy_plan( this->m_PlanBackwardMask );
   fftw_destroy_plan( this->m_PlanBackwardMask2 );
 
-  fftw_destroy_plan( this->m_PlanFilterSquare );
   fftw_destroy_plan( this->m_PlanFilter );
   fftw_destroy_plan( this->m_PlanFilterMask );
   
   fftw_free( this->m_FilterMaskFT2 );
   fftw_free( this->m_FilterMaskFT );
-  fftw_free( this->m_FilterSquareFT );
   fftw_free( this->m_FilterFT );
   fftw_free( this->m_ImageFT );
 }
@@ -100,19 +99,18 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::GetFilteredImageData( co
   this->m_MarginWidth = marginWidth;
 
   memset( this->m_FilterFT, 0, sizeof( fftw_complex ) * this->m_NumberOfPixels );
-  memset( this->m_FilterSquareFT, 0, sizeof( fftw_complex ) * this->m_NumberOfPixels );
   memset( this->m_FilterMaskFT, 0, sizeof( fftw_complex ) * this->m_NumberOfPixels );
 
   this->MakeFilter( sphereRadius, marginWidth );
 
-  const Types::DataItem denom2 = sqrt( this->m_SumFilterSquare - (this->m_SumFilter*this->m_SumFilter) / this->m_SumFilterMask );
+  const Types::DataItem denom2 = sqrt( this->m_SumFilterMask - (this->m_SumFilter*this->m_SumFilter) / this->m_SumFilterMask );
 
   // compute filter kernel FT
   fftw_execute( this->m_PlanFilter );
-  fftw_execute( this->m_PlanFilterSquare );
   fftw_execute( this->m_PlanFilterMask );
   
   // apply FT'ed filter to FT'ed image
+#pragma omp parallel for
   for ( size_t n = 0; n < this->m_NumberOfPixels; ++n )
     {
     this->m_FilterMaskFT2[n][0] = this->m_FilterMaskFT[n][0];
@@ -128,13 +126,15 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::GetFilteredImageData( co
   fftw_execute( this->m_PlanBackwardMask );
   fftw_execute( this->m_PlanBackwardMask2 );
 
+  const double invNumberOfPixels = 1.0 / this->m_NumberOfPixels;
+#pragma omp parallel for
   for ( size_t n = 0; n < this->m_NumberOfPixels; ++n )
     {
     for ( int c = 0; c < 2; ++c )
       {
-      this->m_FilterMaskFT[n][c] /= this->m_NumberOfPixels;
-      this->m_FilterMaskFT2[n][c] /= this->m_NumberOfPixels;
-      this->m_FilterFT[n][c] /= this->m_NumberOfPixels;
+      this->m_FilterMaskFT[n][c] *= invNumberOfPixels;
+      this->m_FilterMaskFT2[n][c] *= invNumberOfPixels;
+      this->m_FilterFT[n][c] *= invNumberOfPixels;
       }
     }
   
@@ -142,6 +142,7 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::GetFilteredImageData( co
   if ( !this->m_FilterResponse )
     this->m_FilterResponse = TypedArray::Create( cmtk::TYPE_ITEM, this->m_NumberOfPixels );
   
+#pragma omp parallel for
   for ( size_t n = 0; n < this->m_NumberOfPixels; ++n )
     {
     const Types::DataItem num = FFTW::Magnitude( this->m_FilterFT[n] ) - (FFTW::Magnitude( this->m_FilterMaskFT[n] ) * this->m_SumFilter / this->m_SumFilterMask );
@@ -163,38 +164,46 @@ cmtk::SphereDetectionNormalizedBipolarMatchedFilterFFT::MakeFilter( const Types:
 			   1 + marginWidth + static_cast<int>( sphereRadius / this->m_PixelSize[1] ), 
 			   1 + marginWidth + static_cast<int>( sphereRadius / this->m_PixelSize[2] ) };
 
-  this->m_SumFilter = this->m_SumFilterMask = this->m_SumFilterSquare = 0;
+  Types::DataItem sumFilter = 0, sumFilterMask = 0;
+
+  const Types::Coordinate sphereRadiusSq = MathUtil::Square( sphereRadius );
+  const Types::Coordinate outerRadiusSq = MathUtil::Square( sphereRadius+marginWidth );
   
   // create a filter kernel for the sphere
+#pragma omp parallel for reduction(+:sumFilter) reduction(+:sumFilterMask)
   for ( int k = 0; k < nRadius[2]; ++k )
+    {
+    const Types::Coordinate dzSq = MathUtil::Square( k * this->m_PixelSize[2] );
     for ( int j = 0; j < nRadius[1]; ++j )
+      {
+      const Types::Coordinate dyzSq = dzSq + MathUtil::Square( j * this->m_PixelSize[1] );
       for ( int i = 0; i < nRadius[0]; ++i )
 	{
-	const cmtk::Types::Coordinate distance = sqrt( cmtk::MathUtil::Square( k * this->m_PixelSize[2] ) + cmtk::MathUtil::Square( j * this->m_PixelSize[1] ) + cmtk::MathUtil::Square( i * this->m_PixelSize[0] ) );
-	if ( distance <= sphereRadius+marginWidth )
+	const Types::Coordinate distanceSq = dyzSq + MathUtil::Square( i * this->m_PixelSize[0] );
+	if ( distanceSq <= outerRadiusSq )
 	  {
-	  cmtk::Types::DataItem value = 1;
-	  if ( (distance > sphereRadius) )
+	  Types::DataItem value = 1;
+	  if ( (distanceSq > sphereRadiusSq) )
 	    {
 	    value = -1;
 	    }
 
-	  if ( value != 0 )
-	    {
-	    for ( int kk = k; kk < this->m_ImageDims[2]; kk += (this->m_ImageDims[2]-1-2*k) )
-	      for ( int jj = j; jj < this->m_ImageDims[1]; jj += (this->m_ImageDims[1]-1-2*j) )
-		for ( int ii = i; ii < this->m_ImageDims[0]; ii += (this->m_ImageDims[0]-1-2*i) )
-		  {
-		  const size_t ofs = ii+this->m_ImageDims[0] * (jj+this->m_ImageDims[1]*kk);
-		  this->m_FilterFT[ofs][0] = value;
-		  this->m_FilterSquareFT[ofs][0] = value*value;
-		  this->m_FilterMaskFT[ofs][0] = 1;
-
-		  this->m_SumFilter += value;
-		  this->m_SumFilterSquare += value * value;
-		  this->m_SumFilterMask += 1;
-		  }
-	    }
+	  for ( int kk = k; kk < this->m_ImageDims[2]; kk += (this->m_ImageDims[2]-1-2*k) )
+	    for ( int jj = j; jj < this->m_ImageDims[1]; jj += (this->m_ImageDims[1]-1-2*j) )
+	      for ( int ii = i; ii < this->m_ImageDims[0]; ii += (this->m_ImageDims[0]-1-2*i) )
+		{
+		const size_t ofs = ii+this->m_ImageDims[0] * (jj+this->m_ImageDims[1]*kk);
+		this->m_FilterFT[ofs][0] = value;
+		this->m_FilterMaskFT[ofs][0] = 1;
+		
+		sumFilter += value;
+		sumFilterMask += 1;
+		}
 	  }
 	}
+      }
+    }
+
+  this->m_SumFilter = sumFilter;
+  this->m_SumFilterMask = sumFilterMask;
 }
