@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3634 $
+//  $Revision: 4835 $
 //
-//  $LastChangedDate: 2011-12-16 14:17:27 -0800 (Fri, 16 Dec 2011) $
+//  $LastChangedDate: 2013-09-12 15:48:47 -0700 (Thu, 12 Sep 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -119,6 +119,23 @@ public:
   { 
     return "string";
   }
+
+  static std::string ValueToString( const std::string& value )
+  {
+    std::ostringstream stream;
+    if ( value.length() )
+      stream << "\"" << value << "\"";
+    else
+      stream << "NONE";
+    return stream.str();
+  }
+
+  static std::string ValueToStringMinimal( const std::string& value )
+  {
+    std::ostringstream stream;
+    stream << value;
+    return stream.str();
+  }
 };
 
 template<>
