@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2012 SRI International
+//  Copyright 2004-2013 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 4306 $
+//  $Revision: 5084 $
 //
-//  $LastChangedDate: 2012-05-04 10:06:54 -0700 (Fri, 04 May 2012) $
+//  $LastChangedDate: 2013-12-03 15:22:17 -0800 (Tue, 03 Dec 2013) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -34,6 +34,7 @@
 
 #include <limits.h>
 #include <stdlib.h>
+#include <errno.h>
 #include <string.h>
 
 #ifdef HAVE_UNISTD_H
@@ -53,6 +54,8 @@
 #  include <windows.h>
 #endif
 
+#include <iostream>
+
 namespace 
 cmtk
 {
@@ -83,7 +86,6 @@ RecursiveMkPrefixDir
 ( const std::string& filename, const int permissions )
 {
   char prefix[PATH_MAX];
-  struct stat buf;
   for ( unsigned i=0; filename[i]; ++i ) 
     {
     if ( (filename[i] == CMTK_PATH_SEPARATOR) || (filename[i] == '/') ) 
@@ -100,14 +102,14 @@ RecursiveMkPrefixDir
 	prefix[i] = '\\';
 	}
 #endif
-      if ( stat( prefix, &buf ) != 0 ) 
-	{
 #ifdef _MSC_VER
-	int result = _mkdir( prefix );
+      const int result = _mkdir( prefix );
 #else
-	int result = mkdir( prefix, permissions );
+      const int result = mkdir( prefix, permissions );
 #endif
-	if ( result ) return result;
+      if ( result && errno != EEXIST && errno != EISDIR ) 
+	{
+	return result;
 	}
       }
     prefix[i] = filename[i];
@@ -139,6 +141,34 @@ GetAbsolutePath( const std::string& relPath )
 #endif
 }
 
+std::string
+Basename( const std::string& path, const std::string& suffix )
+{
+  std::string basename = path;
+
+  // if given, and if present, remove suffix
+  if ( ! suffix.empty() && (basename.length() >= suffix.length() ) )
+    {
+    if ( basename.compare( basename.length() - suffix.length(), suffix.length(), suffix ) )
+      {
+      basename = basename.substr( 0, basename.length() - suffix.length() );
+      }
+    }
+
+  // split into dirname and basename now
+  const size_t separator = basename.rfind( CMTK_PATH_SEPARATOR );
+  if ( separator == std::string::npos )
+    {
+    // no separator, return the whole thing
+    return basename;
+    }
+  else
+    {
+    // seprator - return whatever comes after
+    return basename.substr( separator+1 );
+    }
+}
+
 } // namespace FileUtils
 
 } // namespace cmtk
